# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from kolla_cli.tests.functional.common import KollaCliTest

import logging
import os
import shutil
import unittest

from kolla_cli.api.client import ClientApi
from kolla_cli.common.ansible_inventory import AnsibleInventory
from kolla_cli.common.utils import get_kolla_cli_etc

INV_NAME = 'inventory.json'

CLIENT = ClientApi()
LOG = logging.getLogger(__name__)


class TestFunctional(KollaCliTest):
    """Test description

    This test will look for old version inventory files in the local current
    working directory. If none are found, it will look in the user's home
    directory. If none are found there too, it will print a warning and skip
    the test.

    Old version inventory files must be named inventory.json.v1,
    inventory.json.v2, etc.

    An upgrade test will be run on each old version inventory file that is
    found.
    """
    def test_upgrade(self):
        inv_fpaths = self._find_inv_fpaths()
        if not inv_fpaths:
            LOG.warning('No old version inventory files were found. '
                        'Skipping test.')
        for inv_fpath in inv_fpaths:
            version = self._get_version(inv_fpath)
            self._replace_inventory(inv_fpath)
            self._test_upgrade(version)

    def _get_version(self, inv_fpath):
        try:
            version = int(inv_fpath.split('.v')[1])
        except Exception:
            raise Exception('Invalid version number on old inventory file: %s'
                            % inv_fpath)
        return version

    def _find_inv_fpaths(self):
        """find old version inventories

        Look in order at these locations:
        - current working directory
        - home directory
        """
        fpaths = []
        search_dirs = [os.getcwd(), os.path.expanduser('~')]
        for search_dir in search_dirs:
            fpaths = self._get_inv_fpaths(search_dir)
            if fpaths:
                break
        return fpaths

    def _get_inv_fpaths(self, inv_dir):
        upg_inventory_paths = []
        fnames = os.listdir(inv_dir)
        for fname in fnames:
            if fname.startswith(INV_NAME + '.v'):
                path = os.path.join(inv_dir, fname)
                upg_inventory_paths.append(path)
        return upg_inventory_paths

    def _replace_inventory(self, old_version_inv_path):
        inv_path = os.path.join(get_kolla_cli_etc(),
                                'ansible', 'inventory.json')
        shutil.copyfile(old_version_inv_path, inv_path)

    def _test_upgrade(self, version):
        hostname = 'test_host_upg'

        # This host add will cause the inventory to be upgraded
        CLIENT.host_add([hostname])
        CLIENT.host_remove([hostname])

        # run tests for each version:
        if version <= 1:
            self._test_v1_upgrade()

        if version <= 2:
            self._test_v2_upgrade()

    def _test_v1_upgrade(self):
        # this is a v1 inventory
        # in v1 > v2, ceilometer was added, check that it's there
        # and verify that all ceilometer groups are in the same groups
        # as heat.
        ansible_inventory = AnsibleInventory()
        heat = CLIENT.service_get(['heat'])[0]
        expected_groups = sorted(heat.get_groups())
        ceilometer = ansible_inventory.services['ceilometer']
        expected_services = ceilometer.get_sub_servicenames()
        expected_services.append('ceilometer')
        expected_services = sorted(expected_services)
        services = CLIENT.service_get_all()
        services_found = []

        for service in services:
            servicename = service.get_name()
            if servicename.startswith('ceilometer'):
                groups = sorted(service.get_groups())
                if servicename == 'ceilometer':
                    self.assertEqual(expected_groups, groups,
                                     'groups mismatch between '
                                     'ceilometer '
                                     'and %s' % servicename)
                elif servicename == 'ceilometer-compute':
                    self.assertEqual(['compute'], groups,
                                     'groups mismatch between '
                                     'ceilometer-compute '
                                     'and %s' % servicename)
                else:
                    # sub-services should have no groups (they inherit)
                    self.assertEqual([], groups,
                                     '%s has unexpected groups'
                                     % servicename)
                services_found.append(servicename)

        services_found = sorted(services_found)
        self.assertEqual(expected_services, services_found,
                         'ceilometer subservices mismatch')

    def _test_v2_upgrade(self):
        # this is a v2 inventory. In the v2 to v3 upgrade, all subservices were
        # fixed up to have a parent service
        services = CLIENT.service_get_all()
        servicenames = []
        for service in services:
            servicenames.append(service.name)
            if '-' in service.name:
                # this is a subservice
                parent = service.get_parent()
                self.assertIsNotNone(parent,
                                     'subservice: %s, is missing its parent'
                                     % service.name)

        # ceilometer-alarms were removed in kolla v3.0.1,
        # check that they're gone
        self.assertNotIn('ceilometer-alarm-evaluator', servicenames,
                         'ceilometer-alarm-evaluator still exists.')
        self.assertNotIn('ceilometer-alarm-notifier', servicenames,
                         'ceilometer-alarm-notifier still exists.')

        # aodh and ceph were added in 3.0.1
        self.assertIn('aodh', servicenames, 'aodh not in inventory')
        self.assertIn('ceph', servicenames, 'ceph not in inventory')


if __name__ == '__main__':
    unittest.main()



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from copy import copy
import json
import jsonpickle
import logging
import os
import tempfile
import traceback
import uuid

import kolla_cli.i18n as u

from kolla_cli.api.exceptions import FailedOperation
from kolla_cli.api.exceptions import HostError
from kolla_cli.api.exceptions import InvalidArgument
from kolla_cli.api.exceptions import InvalidConfiguration
from kolla_cli.api.exceptions import MissingArgument
from kolla_cli.api.exceptions import NotInInventory
from kolla_cli.common.ansible_inventory import AnsibleInventory
from kolla_cli.common.host import Host
from kolla_cli.common.host_group import HostGroup
from kolla_cli.common.service import Service
from kolla_cli.common.sshutils import ssh_setup_host
from kolla_cli.common.utils import get_admin_uids
from kolla_cli.common.utils import get_ansible_command
from kolla_cli.common.utils import get_group_vars_dir
from kolla_cli.common.utils import get_host_vars_dir
from kolla_cli.common.utils import get_kolla_cli_etc
from kolla_cli.common.utils import run_cmd
from kolla_cli.common.utils import sync_read_file
from kolla_cli.common.utils import sync_write_file

ANSIBLE_SSH_USER = 'ansible_ssh_user'
ANSIBLE_CONNECTION = 'ansible_connection'
ANSIBLE_BECOME = 'ansible_become'

INVENTORY_PATH = 'ansible/inventory.json'

COMPUTE_GRP_NAME = 'compute'

# these groups cannot be deleted, they are required by kolla
PROTECTED_GROUPS = [COMPUTE_GRP_NAME]

LOG = logging.getLogger(__name__)


def remove_temp_inventory(path):
    # type: (str) -> None
    """remove temp inventory file and its parent directory"""
    if path:
        if os.path.exists(path):
            os.remove(path)
        dirpath = os.path.dirname(path)
        if os.path.exists(dirpath):
            os.rmdir(dirpath)


class Inventory(object):
    class_version = 4
    """class version history

    4: (v4.0.1):
        removed concept of sub-services (not backward compatible)
    3: (v3.0.1):
        added aodh, ceph
        fix to ensure all sub-services have service as parent

    2: (v2.1.1) added ceilometer

    1: (v2.0.1) initial release
    """
    def __init__(self, inventory_path=None):
        self._groups = {}           # kv = name:object
        self._hosts = {}            # kv = name:object
        self._services = {}         # kv = name:object
        self.vars = {}
        self.version = self.__class__.class_version
        self.remote_mode = True

        # initialize the inventory to its defaults as they are
        # described in the kolla-ansible all-in-one inventory file.
        self.import_inventory(inventory_path)

    def upgrade(self):
        # check for new services or subservices in the all-in-one file
        # leaving in this hook but no upgrade from <4 to 4 is supported
        # so yanking out all upgrade logic
        self._upgrade_services()

        # update the version and save upgraded inventory file
        self.version = self.__class__.class_version
        Inventory.save(self)

    def _upgrade_services(self):
        ansible_inventory = AnsibleInventory()

        # add new services
        for servicename, service in ansible_inventory.services.items():
            if servicename not in self._services:
                self._services[servicename] = service
            else:
                # If Service newly enabled/disabled, take the latest
                if self._services[servicename].is_supported() != \
                        service.is_supported():
                    self._services[servicename].set_supported(
                        service.is_supported())

                # If Service group changed, take the latest
                if sorted(self._services[servicename].get_groupnames()) != \
                        sorted(service.get_groupnames()):
                    self._services[servicename].set_groupnames(
                        service.get_groupnames())

        # remove obsolete services
        for servicename in copy(self._services).keys():
            if servicename not in ansible_inventory.services:
                self.delete_service(servicename)

    @staticmethod
    def load():
        """load the inventory from a pickle file"""
        inventory_path = os.path.join(get_kolla_cli_etc(), INVENTORY_PATH)
        data = ''
        try:
            if os.path.exists(inventory_path):
                data = sync_read_file(inventory_path)

            if data.strip():
                inventory = jsonpickle.decode(data)

                # upgrade version handling
                if inventory.version != inventory.class_version:
                    inventory.upgrade()
            else:
                inventory = Inventory()
                Inventory.save(inventory)
        except Exception:
            raise FailedOperation(
                u._('Loading inventory failed. : {error}')
                .format(error=traceback.format_exc()))
        return inventory

    @staticmethod
    def save(inventory):
        """Save the inventory in a pickle file"""
        inventory_path = os.path.join(get_kolla_cli_etc(), INVENTORY_PATH)
        try:
            # multiple trips thru json to render a readable inventory file
            data = jsonpickle.encode(inventory)
            data_str = json.loads(data)
            pretty_data = json.dumps(data_str, indent=4)
            sync_write_file(inventory_path, pretty_data)

        except Exception as e:
            raise FailedOperation(
                u._('Saving inventory failed. : {error}')
                .format(error=str(e)))

    def get_hosts(self):
        return self._hosts.values()

    def get_hostnames(self):
        return list(self._hosts.keys())

    def get_host(self, hostname):
        host = None
        if hostname in self._hosts:
            host = self._hosts[hostname]
        return host

    def add_host(self, hostname, groupname=None):
        """add host

        if groupname is none, create a new host
        if group name is not none, add host to group
        """
        if groupname and groupname not in self._groups:
            raise NotInInventory(u._('Group'), groupname)

        if groupname and hostname not in self._hosts:
            # if a groupname is specified, the host must already exist
            raise NotInInventory(u._('Host'), hostname)

        if not groupname and not self.remote_mode and len(self._hosts) >= 1:
            raise InvalidConfiguration(
                u._('Cannot have more than one host when in local deploy '
                    'mode.'))

        changed = False
        # create new host if it doesn't exist
        host = Host(hostname)
        if hostname not in self.get_hostnames():
            # a new host is being added to the inventory
            changed = True
            self._hosts[hostname] = host

        # a host is to be added to an existing group
        elif groupname:
            group = self._groups[groupname]
            if hostname not in group.get_hostnames():
                changed = True
                group.add_host(host)
        return changed

    def remove_all_hosts(self):
        """remove all hosts."""
        hostnamess = self.get_hostnames()
        for hostname in hostnamess:
            self.remove_host(hostname)

    def remove_host(self, hostname, groupname=None):
        """remove host

        if groupname is none, delete host
        if group name is not none, remove host from group
        """
        changed = False
        if groupname and groupname not in self._groups:
            raise NotInInventory(u._('Group'), groupname)

        if hostname not in self._hosts:
            return changed

        changed = True
        host = self._hosts[hostname]
        groups = self.get_groups(host)
        for group in groups:
            if not groupname or groupname == group.name:
                group.remove_host(host)

        if not groupname:
            host_vars = os.path.join(get_host_vars_dir(), hostname)
            if os.path.exists(host_vars):
                os.remove(host_vars)
            del self._hosts[hostname]
        return changed

    def setup_hosts(self, hosts_info):
        """setup multiple hosts

        hosts_info is a dict of format:
            {'hostname1': {
                'password': password
                'uname': user_name}}

        The uname entry is optional.
        """
        failed_hosts = {}
        for hostname, host_info in hosts_info.items():
            host = self.get_host(hostname)
            if not host:
                failed_hosts[hostname] = u._("Host doesn't exist.")
                continue
            if not host_info or 'password' not in host_info:
                failed_hosts[hostname] = u._('No password in yml file.')
                continue
            passwd = host_info['password']

            # NOTE(caoyuan): The type of password must be string,
            # convert to string if the password is int.
            if not isinstance(passwd, str):
                passwd = str(passwd)
            uname = None
            if 'uname' in host_info:
                uname = host_info['uname']
            try:
                self.setup_host(hostname, passwd, uname)
            except Exception as e:
                failed_hosts[hostname] = '%s' % e
        if failed_hosts:
            summary = '\n'
            for hostname, err in failed_hosts.items():
                summary = summary + '- %s: %s\n' % (hostname, err)
            raise HostError(
                u._('Not all hosts were set up. : {reasons}')
                .format(reasons=summary))
        else:
            LOG.info(u._LI('All hosts were successfully set up.'))

    def setup_host(self, hostname, password, uname=None):
        try:
            LOG.info(
                u._LI('Starting setup of host ({host}).')
                .format(host=hostname))
            check_ok, _ = self.ssh_check_host(hostname)
            if check_ok:
                LOG.info(u._LI('Host ({host}) is already setup.')
                         .format(host=hostname))
            else:
                # host needs setup
                ssh_setup_host(hostname, password, uname)
                check_ok, msg = self.ssh_check_host(hostname)
                if not check_ok:
                    raise Exception(u._('Post-setup ssh check failed. {err}')
                                    .format(err=msg))
                LOG.info(u._LI('Host ({host}) setup succeeded.')
                         .format(host=hostname))
        except Exception as e:
            raise HostError(
                u._('Host ({host}) setup failed : {error}')
                .format(host=hostname, error=str(e)))
        return True

    def ssh_check_hosts(self, hostnames):
        """ssh check for hosts

        return {hostname: {'success': True|False,
                           'msg': message}}
        """
        summary = {}
        for hostname in hostnames:
            is_ok, msg = self.ssh_check_host(hostname)
            summary[hostname] = {}
            summary[hostname]['success'] = is_ok
            summary[hostname]['msg'] = msg
        return summary

    def ssh_check_host(self, hostname):
        err_msg, output = self.run_ansible_command('-m ping', hostname)
        is_ok = True
        if err_msg:
            is_ok = False
            msg = (
                u._('Host ({host}) ssh check failed. : {error} {message}')
                .format(host=hostname, error=err_msg, message=output))
        else:
            msg = (u._LI('Host ({host}) ssh check succeeded.')
                   .format(host=hostname))
        return is_ok, msg

    def run_ansible_command(self, ansible_command, hostname):
        output = None
        command_string = '%s -vvv' % \
            get_ansible_command()
        gen_file_path = self.create_json_gen_file()
        cmd = '%s %s -i %s %s' % (command_string, hostname, gen_file_path,
                                  ansible_command)
        try:
            err_msg, output = run_cmd(cmd, False)
        except Exception as e:
            err_msg = str(e)
        finally:
            self.remove_json_gen_file(gen_file_path)
        return err_msg, output

    def add_group(self, groupname):

        # Group names cannot overlap with service names:
        if groupname in self._services:
            raise InvalidArgument(
                u._('Invalid group name. A service name '
                    'cannot be used for a group name.'))

        if groupname not in self._groups:
            self._groups[groupname] = HostGroup(groupname)

        group = self._groups[groupname]

        group.set_remote(self.remote_mode)

        return group

    def remove_group(self, groupname):
        if groupname in PROTECTED_GROUPS:
            raise InvalidArgument(
                u._('Cannot remove {group} group. It is required by kolla.')
                .format(group=groupname))

        # remove group from services
        for service in self._services.values():
            service.remove_groupname(groupname)

        group_vars = os.path.join(get_group_vars_dir(), groupname)
        if os.path.exists(group_vars) and groupname != '__GLOBAL__':
            os.remove(group_vars)

        if groupname in self._groups:
            del self._groups[groupname]

    def get_group(self, groupname):
        group = None
        if groupname in self._groups:
            group = self._groups[groupname]
        return group

    def get_groupnames(self):
        return list(self._groups.keys())

    def get_groups(self, host=None):
        """return all groups containing host

        if hosts is none, return all groups in inventory
        """
        groups = []
        if not host:
            groups = self._groups.values()

        else:
            for group in self._groups.values():
                if host.name in group.get_hostnames():
                    groups.append(group)
        return groups

    def get_host_groups(self):
        """return { hostname : [groupnames] }"""

        host_groups = {}
        for host in self._hosts.values():
            host_groups[host.name] = []
            groups = self.get_groups(host)
            for group in groups:
                host_groups[host.name].append(group.name)
        return host_groups

    def get_group_services(self):
        """get groups and their services

        return { groupname: [servicenames] }
        """

        group_services = {}

        for group in self.get_groups():
            group_services[group.name] = []

        for svc in self.get_services():
            for groupname in svc.get_groupnames():
                group_services[groupname].append(svc.name)
        return group_services

    def get_group_hosts(self):
        """return { groupname : [hostnames] }"""
        group_hosts = {}
        for group in self.get_groups():
            group_hosts[group.name] = []
            for hostname in group.get_hostnames():
                group_hosts[group.name].append(hostname)
        return group_hosts

    def create_service(self, servicename):
        if servicename not in self._services:
            service = Service(servicename)
            self._services[servicename] = service
        return self._services[servicename]

    def delete_service(self, servicename):
        # remove references to this service from all parent / child services
        if servicename in self._services:
            service = self._services[servicename]
            for parentname in service.get_parentnames():
                parent = self._services[parentname]
                parent.remove_childname(servicename)
            for childname in service.get_childnames():
                child = self._services[childname]
                child.remove_parentname(servicename)

            # then remove the service itself
            del self._services[servicename]

    def get_services(self, client_filter=False):
        if client_filter:
            services = self._client_filtered_service_dict()
        else:
            services = self._services

        return services.values()

    def get_service(self, servicename, client_filter=False):
        service = None
        if client_filter:
            services = self._client_filtered_service_dict()
        else:
            services = self._services

        if servicename in services:
            service = services[servicename]
        return service

    def add_group_to_service(self, groupname, servicename):
        if groupname not in self._groups:
            raise NotInInventory(u._('Group'), groupname)
        if servicename in self._services:
            service = self.get_service(servicename)
            service.add_groupname(groupname)
        else:
            raise NotInInventory(u._('Service'), servicename)

    def remove_group_from_service(self, groupname, servicename):
        if groupname not in self._groups:
            raise NotInInventory(u._('Group'), groupname)
        if servicename in self._services:
            service = self.get_service(servicename)
            service.remove_groupname(groupname)
        else:
            raise NotInInventory(u._('Service'), servicename)

    def set_deploy_mode(self, remote_flag):
        if not remote_flag and len(self._hosts) > 1:
            raise InvalidConfiguration(
                u._('Cannot set local deploy mode when multiple hosts exist.'))
        self.remote_mode = remote_flag

        for group in self.get_groups():
            group.set_remote(remote_flag)

    def get_ansible_json(self, inventory_filter=None):
        """generate json inventory for ansible

        The hosts and groups added to the json output for ansible will be
        filtered by the hostnames and groupnames in the deploy filters.
        This allows a more targeted deploy to a specific set of hosts or
        groups.

        typical ansible json format:
            {'group': {
                'hosts': ['192.168.28.71', '192.168.28.72'],
                'vars': {

                    'ansible_ssh_user': 'johndoe',
                    'ansible_ssh_private_key_file': '~/.ssh/mykey',
                    'example_variable': 'value'},

                'children': [ 'marietta', '5points' ]},

             '_meta': {
                 'hostvars': {

                     '192.168.28.71': {
                         'host_specific_var': 'bar'},

                     '192.168.28.72': {
                         'host_specific_var': 'foo'}}}}
        """

        jdict = {}

        # if no filter provided, use all groups, all hosts
        deploy_hostnames = self.get_hostnames()
        deploy_groupnames = self.get_groupnames()
        if inventory_filter:
            if 'deploy_hosts' in inventory_filter:
                deploy_hostnames = inventory_filter['deploy_hosts']
            if 'deploy_groups' in inventory_filter:
                deploy_groupnames = inventory_filter['deploy_groups']

        # add hostgroups
        for group in self.get_groups():
            jdict[group.name] = {}
            jdict[group.name]['hosts'] = []

            if group.name in deploy_groupnames:
                jdict[group.name]['hosts'] = \
                    self._filter_hosts(group.get_hostnames(), deploy_hostnames)
            jdict[group.name]['children'] = []
            jdict[group.name]['vars'] = group.get_vars()

        # add all services, what groups they are in and their parents
        for service in self.get_services():
            jdict[service.name] = {}
            groups_and_parents = []
            groups_and_parents.extend(service.get_groupnames())
            groups_and_parents.extend(service.get_parentnames())
            jdict[service.name]['children'] = groups_and_parents

        # temporarily create group containing all hosts. this is needed for
        # ansible commands that are performed on hosts not yet in groups.
        group = self.add_group('__GLOBAL__')
        jdict[group.name] = {}
        jdict[group.name]['hosts'] = deploy_hostnames
        jdict[group.name]['vars'] = group.get_vars()
        self.remove_group(group.name)

        # process hosts vars
        jdict['_meta'] = {}
        jdict['_meta']['hostvars'] = {}
        for hostname in deploy_hostnames:
            host = self.get_host(hostname)
            if host:
                jdict['_meta']['hostvars'][hostname] = host.get_vars()
        return json.dumps(jdict)

    def _filter_hosts(self, initial_hostnames, deploy_hostnames):
        """filter out hosts not in deploy hosts

        Must preserve the ordering of hosts in the group.
        """
        filtered_hostnames = []
        for hostname in initial_hostnames:
            if hostname in deploy_hostnames:
                filtered_hostnames.append(hostname)
        return filtered_hostnames

    def create_json_gen_file(self, inventory_filter=None):
        """create json inventory file using filter ({})

        The inventory will be placed in a directory in /tmp,
        with the directory name of form kolla_uuid.py,
        where uuid is a unique deployment id.

        return path to filtered json generator file
        """
        json_out = self.get_ansible_json(inventory_filter)

        deploy_id = str(uuid.uuid4())
        dirname = 'kolla_%s' % deploy_id
        dirpath = os.path.join(tempfile.gettempdir(), dirname)
        os.mkdir(dirpath, 0o775)
        _, gid = get_admin_uids()
        json_gen_path = os.path.join(dirpath, 'temp_inventory.py')

        with open(json_gen_path, 'w') as json_gen_file:
            json_gen_file.write('#!/usr/bin/env python\n')
            # the quotes here are significant. The json_out has double quotes
            # embedded in it so single quotes are needed to wrap it.
            json_gen_file.write("print('%s')" % json_out)

        # set executable by group
        os.chmod(json_gen_path, 0o555)  # nosec
        return json_gen_path

    def remove_json_gen_file(self, path):
        remove_temp_inventory(path)

    def validate_hostnames(self, hostnames):
        if not hostnames:
            raise MissingArgument(u._('Host name(s)'))
        invalid_hosts = []
        for hostname in hostnames:
            if hostname not in self._hosts:
                invalid_hosts.append(hostname)
        if invalid_hosts:
            raise NotInInventory(u._('Host'), invalid_hosts)

    def validate_groupnames(self, groupnames):
        if not groupnames:
            raise MissingArgument(u._('Group name(s)'))
        invalid_groups = []
        for groupname in groupnames:
            if groupname not in self._groups:
                invalid_groups.append(groupname)
        if invalid_groups:
            raise NotInInventory(u._('Group'), invalid_groups)

    def validate_servicenames(self, servicenames, client_filter=False):
        if not servicenames:
            raise MissingArgument(u._('Service name(s)'))
        invalid_services = []
        if client_filter:
            services = self._client_filtered_service_dict()
        else:
            services = self._services

        for service_name in servicenames:
            if service_name not in services:
                invalid_services.append(service_name)
        if invalid_services:
            raise NotInInventory(u._('Service'), invalid_services)

    def _client_filtered_service_dict(self):
        """Filters out any unsupported services

        :return: filtered dictionary
        """
        filtered_service_dict = {}
        for service in self._services.values():
            if service.is_supported():
                filtered_service_dict[service.name] = service
        return filtered_service_dict

    def import_inventory(self, inventory_path):
        ansible_inventory = AnsibleInventory(inventory_path)
        for groupname in ansible_inventory.groups:
            self.add_group(groupname)
        for servicename, service in ansible_inventory.services.items():
            self._services[servicename] = service



#!/usr/bin/env python
# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import shutil
import sys
import tarfile
import tempfile
import traceback

from kolla_cli.api.client import ClientApi

tar_file_descr = None

CLIENT = ClientApi()

LOGDIR = '/tmp/container_logs'


def get_logs_from_host(host):
    try:
        service_names = []
        services = CLIENT.service_get_all()
        for service in services:
            service_names.append(service.name)

        print('Adding container logs from host: %s' % host)
        CLIENT.support_get_logs(service_names, host, LOGDIR)
    except Exception as e:
        print('Error getting logs on host: %s: %s' % (host, str(e)))


def dump_kolla_info():
    print('Getting kolla client logs')
    dump_path = None
    try:
        dump_path = CLIENT.support_dump('/tmp')
        tar_file_descr.add(dump_path)
    except Exception:
        print('ERROR: running dump command %s' % traceback.format_exc())
    finally:
        if dump_path and os.path.exists(dump_path):
            os.remove(dump_path)


def main():
    """collect docker logs from servers

    $ command is $ log_collector.py <all | host1[,host2,host3...]>
    """
    global tar_file_descr

    help_msg = 'Usage: log_collector.py <all | host1[,host2,host3...]>'
    hosts = []
    if len(sys.argv) == 2:
        if '-h' == sys.argv[1] or '--help' == sys.argv[1]:
            print(help_msg)
            sys.exit(0)
        elif 'all' == sys.argv[1]:
            # get logs from all hosts
            hosts = []
            host_objs = CLIENT.host_get_all()
            for host_obj in host_objs:
                hosts.append(host_obj.name)
        else:
            # get logs from specified hosts
            hostnames = sys.argv[1].split(',')
            for host in hostnames:
                if host not in hosts:
                    hosts.append(host)
    else:
        print(help_msg)
        sys.exit(1)

    # open tar file for storing logs
    fd, tar_path = tempfile.mkstemp(prefix='kolla_support_logs_',
                                    suffix='.tgz')
    os.close(fd)  # avoid fd leak

    with tarfile.open(tar_path, 'w:gz') as tar_file_descr:
        # clear out old logs
        if os.path.exists(LOGDIR):
            shutil.rmtree(LOGDIR)
        os.mkdir(LOGDIR)

        # gather logs from selected hosts
        try:
            for host in hosts:
                get_logs_from_host(host)

            # tar up all the container logs
            tar_file_descr.add(LOGDIR, arcname='container_logs')
        finally:
            # remove uncompressed logs
            if os.path.exists(LOGDIR):
                shutil.rmtree(LOGDIR)

        # gather dump output from kolla-cli
        dump_kolla_info()

    print('Log collection complete. Logs are at %s' % tar_path)


if __name__ == '__main__':
    main()



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import os
import unittest

import kolla_cli.api.properties

from kolla_cli.api.client import ClientApi
from kolla_cli.tests.functional.common import KollaCliTest

CLIENT = ClientApi()


class TestFunctional(KollaCliTest):

    def test_config_reset(self):
        # test global property reset
        # set a property and make sure it was set correctly
        property_dict = {'test': 'test'}
        CLIENT.property_set(property_dict)
        fetched_properties = CLIENT.property_get()
        fetched_dict = self._properties_to_dict(fetched_properties)
        self.assertIs(self._in_dict(property_dict, fetched_dict), True,
                      'property set failed')

        # now clear the config and make sure the property we just
        # set is now gone
        CLIENT.config_reset()
        fetched_properties = CLIENT.property_get()
        fetched_dict = self._properties_to_dict(fetched_properties)
        self.assertIs(self._in_dict(property_dict, fetched_dict), False,
                      'global property reset config failed')

        # test host property reset
        host_list = ['test']
        CLIENT.host_add(host_list)
        CLIENT.property_set(property_dict,
                            kolla_cli.api.properties.HOST_TYPE)
        fetched_properties = CLIENT.property_get(
            kolla_cli.api.properties.HOST_TYPE, host_list)
        fetched_dict = self._properties_to_dict(fetched_properties)
        self.assertIs(self._in_dict(property_dict, fetched_dict), True,
                      'host property set failed')

        CLIENT.config_reset()
        # need to add back in the host 'test' or the property
        # get call will fail after a reset
        CLIENT.host_add(host_list)
        fetched_properties = CLIENT.property_get(
            kolla_cli.api.properties.HOST_TYPE, host_list)
        fetched_dict = self._properties_to_dict(fetched_properties)
        self.assertIs(self._in_dict(property_dict, fetched_dict), False,
                      'host property reset config failed')

        # test group property reset
        group_list = ['control']
        CLIENT.property_set(property_dict,
                            kolla_cli.api.properties.GROUP_TYPE)
        fetched_properties = CLIENT.property_get(
            kolla_cli.api.properties.GROUP_TYPE, group_list)
        fetched_dict = self._properties_to_dict(fetched_properties)
        self.assertIs(self._in_dict(property_dict, fetched_dict), True,
                      'group property set failed')

        CLIENT.config_reset()
        fetched_properties = CLIENT.property_get(
            kolla_cli.api.properties.GROUP_TYPE, group_list)
        fetched_dict = self._properties_to_dict(fetched_properties)
        self.assertIs(self._in_dict(property_dict, fetched_dict), False,
                      'group property reset config failed')

        # test host reset
        # add a host and make sure it was added correctly
        host_list = ['test']
        CLIENT.host_add(host_list)
        fetched_hosts = CLIENT.host_get_all()
        fetched_list = self._hosts_to_list(fetched_hosts)
        self.assertIs(set(host_list).issubset(fetched_list), True,
                      'host set failed')

        # now clear the config and make sure the host we just
        # added is now gone
        CLIENT.config_reset()
        fetched_hosts = CLIENT.host_get_all()
        fetched_list = self._hosts_to_list(fetched_hosts)
        self.assertIs(set(host_list).issubset(fetched_list), False,
                      'inventory reset config failed')

        # need to populate the password file or many other tests will fail
        CLIENT.password_init()

    def test_config_import_inventory(self):
        # test config import of a different inventory file
        expected_group_names = ['chipmunk', 'aardvark']
        test_inventory_path = os.path.join(
            os.getcwd(), 'kolla_cli', 'tests', 'functional',
            'inventory_test_file')
        CLIENT.config_import_inventory(file_path=test_inventory_path)
        groups = CLIENT.group_get_all()
        self.assertEqual(len(groups), len(expected_group_names))
        for group in groups:
            self.assertIn(group.name, expected_group_names)

        # need to reset the inventory back to its defaults
        CLIENT.config_reset()

    @staticmethod
    def _properties_to_dict(props):
        property_dict = {}
        for prop in props:
            property_dict[prop.name] = prop.value
        return property_dict

    @staticmethod
    def _hosts_to_list(hosts):
        host_list = []
        for host in hosts:
            host_list.append(host.name)
        return host_list

    @staticmethod
    def _in_dict(base, target):
        base_keys = base.keys()
        target_keys = target.keys()
        if set(base_keys).issubset(target_keys) is False:
            return False

        for key in base.keys():
            target_value = target.get(key, None)
            if target_value != base[key]:
                return False

        return True


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2018 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from kolla_cli.tests.unit.common import KollaCliUnitTest


class TestUnit(KollaCliUnitTest):
    @mock.patch('kolla_cli.api.client.ClientApi.host_add')
    @mock.patch('kolla_cli.shell.KollaCli._is_inventory_present',
                return_value=True)
    def test_host_add(self, _, mock_add):
        hostname = 'foo'
        ret = self.run_cli_command('host add %s' % hostname)
        self.assertEqual(ret, 0)
        mock_add.assert_called_once_with([hostname])

    @mock.patch('kolla_cli.api.client.ClientApi.host_remove')
    @mock.patch('kolla_cli.shell.KollaCli._is_inventory_present',
                return_value=True)
    def test_host_remove(self, _, mock_remove):
        hostname = 'foo'
        ret = self.run_cli_command('host remove %s' % hostname)
        self.assertEqual(ret, 0)
        mock_remove.assert_called_once_with([hostname])

    @mock.patch('kolla_cli.api.client.ClientApi.host_get_all')
    @mock.patch('kolla_cli.api.client.ClientApi.host_get')
    @mock.patch('kolla_cli.shell.KollaCli._is_inventory_present',
                return_value=True)
    def test_host_list(self, _, mock_get, mock_get_all):
        # get all hosts
        mock_get_all.return_value = []
        ret = self.run_cli_command('host list')
        self.assertEqual(ret, 0)
        mock_get_all.assert_called_once_with()

        # get a specific host
        hostname = 'foo'
        mock_get.return_value = []
        ret = self.run_cli_command('host list %s' % hostname)
        self.assertEqual(ret, 0)
        mock_get.assert_called_once_with([hostname])

    @mock.patch('kolla_cli.commands.host.HostDestroy._is_ok_to_delete_data',
                return_value='y')
    @mock.patch('kolla_cli.common.ansible.job.AnsibleJob.get_status')
    @mock.patch('kolla_cli.api.client.ClientApi.host_get_all')
    @mock.patch('kolla_cli.api.client.ClientApi.host_destroy')
    @mock.patch('kolla_cli.shell.KollaCli._is_inventory_present',
                return_value=True)
    def test_host_destroy(self, _, mock_destroy, mock_get_all,
                          mock_get_status, mock_prompt):
        hostname = 'foo'
        mock_get_all.return_value = [self.get_fake_host(hostname)]
        mock_destroy.return_value = self.get_fake_job()
        mock_get_status.return_value = 0

        # default destroy hostname
        ret = self.run_cli_command('host destroy %s' % hostname)
        self.assertEqual(ret, 0)
        mock_destroy.assert_called_once_with([hostname], 'kill', 1,
                                             False, False)
        # destroy all
        mock_destroy.reset_mock()
        ret = self.run_cli_command('host destroy all')
        self.assertEqual(ret, 0)
        mock_destroy.assert_called_once_with([hostname], 'kill', 1,
                                             False, False)
        # destroy --stop
        mock_destroy.reset_mock()
        ret = self.run_cli_command('host destroy %s --stop' % hostname)
        self.assertEqual(ret, 0)
        mock_destroy.assert_called_once_with([hostname], 'stop', 1,
                                             False, False)
        # destroy --includedata
        mock_destroy.reset_mock()
        ret = self.run_cli_command('host destroy %s --includedata' % hostname)
        self.assertEqual(ret, 0)
        mock_destroy.assert_called_once_with([hostname], 'kill', 1,
                                             True, False)

        # destroy --removeimages
        mock_destroy.reset_mock()
        ret = self.run_cli_command('host destroy %s --removeimages'
                                   % hostname)
        self.assertEqual(ret, 0)
        mock_destroy.assert_called_once_with([hostname], 'kill', 1,
                                             False, True)

    @mock.patch('kolla_cli.commands.host.LOG.info')
    @mock.patch('kolla_cli.api.client.ClientApi.host_get_all')
    @mock.patch('kolla_cli.api.client.ClientApi.host_ssh_check')
    @mock.patch('kolla_cli.shell.KollaCli._is_inventory_present',
                return_value=True)
    def test_host_ssh_check(self, _, mock_ssh_check, mock_get_all, mock_log):
        hostname = 'foo'
        check_ok_response = {hostname: {'success': True}}
        check_bad_response = {hostname: {'success': False, 'msg': 'FAILED'}}
        mock_get_all.return_value = [self.get_fake_host(hostname)]

        # host check hostname (success)
        mock_ssh_check.return_value = check_ok_response
        ret = self.run_cli_command('host check %s' % hostname)
        self.assertEqual(ret, 0)
        mock_ssh_check.assert_called_once_with([hostname])
        mock_log.assert_called_once_with('Host %s: success ' % hostname)

        # host check all (success)
        mock_ssh_check.reset_mock()
        mock_log.reset_mock()
        mock_ssh_check.return_value = check_ok_response
        ret = self.run_cli_command('host check all')
        self.assertEqual(ret, 0)
        mock_ssh_check.assert_called_once_with([hostname])
        mock_log.assert_called_once_with('Host %s: success ' % hostname)

        # host check hostname (fail)
        mock_ssh_check.reset_mock()
        mock_log.reset_mock()
        mock_ssh_check.return_value = check_bad_response
        ret = self.run_cli_command('host check %s' % hostname)
        self.assertEqual(ret, 1)
        mock_ssh_check.assert_called_once_with([hostname])
        mock_log.assert_called_once_with('Host %s: failed- FAILED' % hostname)

    @mock.patch('kolla_cli.commands.host.HostSetup._get_yml_data')
    @mock.patch('getpass.getpass')
    @mock.patch('kolla_cli.commands.host.ClientApi.host_ssh_check')
    @mock.patch('kolla_cli.commands.host.ClientApi.host_setup')
    @mock.patch('kolla_cli.api.client.ClientApi.host_get_all')
    @mock.patch('kolla_cli.shell.KollaCli._is_inventory_present',
                return_value=True)
    def test_host_setup(self, _, mock_get_all, mock_setup, mock_ssh_check,
                        mock_passwd, mock_yml):
        password = 'PASSWORD'
        hostname = 'foo'
        mock_get_all.return_value = [self.get_fake_host(hostname)]
        mock_passwd.return_value = password

        # single host setup (host not yet setup)
        mock_ssh_check.return_value = {hostname: {'success': False}}
        ret = self.run_cli_command('host setup %s' % hostname)
        self.assertEqual(ret, 0)
        mock_ssh_check.assert_called_once_with([hostname])
        mock_setup.assert_called_once_with({hostname: {'password': password}})

        # single host setup --insecure (host already setup)
        mock_ssh_check.reset_mock()
        mock_setup.reset_mock()
        mock_ssh_check.return_value = {hostname: {'success': True}}
        ret = self.run_cli_command('host setup %s --insecure %s'
                                   % (hostname, password))
        self.assertEqual(ret, 0)
        mock_ssh_check.assert_called_once_with([hostname])
        mock_setup.assert_not_called()

        # multi-host setup
        mock_ssh_check.reset_mock()
        mock_setup.reset_mock()
        fake_path = '/bogus'
        mock_yml.return_value = {hostname: {'password': password}}
        ret = self.run_cli_command('host setup --file %s' % fake_path)
        self.assertEqual(ret, 0)
        mock_setup.assert_called_once_with({hostname: {'password': password}})
        mock_yml.assert_called_once_with(fake_path)
        mock_ssh_check.assert_not_called()



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from copy import copy

from kolla_cli.common.inventory import Inventory
from kolla_cli.common.utils import check_arg
from kolla_cli.common.utils import safe_decode
import kolla_cli.i18n as u


MYPY = False
if MYPY:
    from typing import List  # noqa


class GroupApi(object):

    def group_add(self, groupnames):
        # type: (List[str]) -> None
        """Add groups to the inventory

        :param groupnames: names of the groups to add to the inventory
        :type groupnames: list of strings

        """
        check_arg(groupnames, u._('Group names'), list)
        groupnames = safe_decode(groupnames)

        inventory = Inventory.load()
        for groupname in groupnames:
            inventory.add_group(groupname)
        Inventory.save(inventory)

    def group_remove(self, groupnames):
        # type: (List[str]) -> None
        """Remove groups from the inventory

        :param groupnames: names of the groups to remove from the inventory
        :type groupnames: list of strings
        """
        check_arg(groupnames, u._('Group names'), list)
        groupnames = safe_decode(groupnames)

        inventory = Inventory.load()
        for groupname in groupnames:
            inventory.remove_group(groupname)
        Inventory.save(inventory)

    def group_get_all(self):
        # type: () -> List[Group]
        """Get all groups in the inventory

        :return: groups
        :rtype: list of Group objects
        """
        return self._get_groups([], get_all=True)

    def group_get(self, groupnames):
        # type: (List[str]) -> List[Group]
        """Get selected groups in the inventory

        :param groupnames: names of groups to be read
        :type groupnames: list of strings
        :return: groups
        :rtype: list of Group objects
        """
        check_arg(groupnames, u._('Group names'), list)
        groupnames = safe_decode(groupnames)
        return self._get_groups(groupnames)

    def _get_groups(self, groupnames, get_all=False):
        # type: (List[str], bool) -> List[Group]
        groups = []
        inventory = Inventory.load()
        if get_all:
            groupnames = inventory.get_groupnames()
        else:
            inventory.validate_groupnames(groupnames)

        group_services = inventory.get_group_services()
        for groupname in groupnames:
            inv_group = inventory.get_group(groupname)
            group = Group(groupname,
                          group_services[groupname],
                          inv_group.get_hostnames())
            groups.append(group)
        return groups


class Group(object):
    def __init__(self, groupname, servicenames, hostnames):
        # type: (str, List[str], List[str]) -> None
        self.name = groupname
        self._servicenames = servicenames
        self._hostnames = hostnames

    def get_name(self):
        # type: () -> str
        """Get name

        :return: group name
        :rtype: string
        """
        return self.name

    def get_services(self):
        # type: () -> List[str]
        """Get names of services associated with this group.

        :return: service names
        :rtype: list of strings
        """
        return copy(self._servicenames)

    def add_service(self, servicename):
        # type: (str) -> None
        """Add service to group

        :param servicename: name of the service to add to the group
        :type servicename: string
        """
        check_arg(servicename, u._('Service name'), str)
        servicename = safe_decode(servicename)
        inventory = Inventory.load()
        inventory.validate_servicenames([servicename], client_filter=True)

        group_services = inventory.get_group_services()
        self._servicenames = group_services[self.name]
        if servicename not in self._servicenames:
            # service not associated with group, add it
            inventory.add_group_to_service(self.name, servicename)
            self._servicenames.append(servicename)
            Inventory.save(inventory)

    def remove_service(self, servicename):
        # type: (str) -> None
        """Remove service from group

        :param servicename: name of the service to remove from the group
        :type servicename: string

        """
        check_arg(servicename, u._('Service name'), str)
        servicename = safe_decode(servicename)
        inventory = Inventory.load()
        inventory.validate_servicenames([servicename], client_filter=True)

        group_services = inventory.get_group_services()
        self._servicenames = group_services[self.name]
        if servicename in self._servicenames:
            # service is associated with group, remove it
            inventory.remove_group_from_service(self.name, servicename)
            self._servicenames.remove(servicename)
            Inventory.save(inventory)

    def get_hosts(self):
        # type: () -> List[str]
        """Get names of hosts associated with this group.

        :return: host names
        :rtype: list of strings
        """
        return copy(self._hostnames)

    def add_host(self, hostname):
        # type: (str) -> None
        """Add host to group

        :param hostname: name of the host to add to the group
        :type hostname: string

        """
        check_arg(hostname, u._('Host name'), str)
        hostname = safe_decode(hostname)
        inventory = Inventory.load()
        inventory.validate_hostnames([hostname])

        group = inventory.get_group(self.name)
        self._hostnames = group.get_hostnames()
        if hostname not in self._hostnames:
            # host not associated with group, add it
            inventory.add_host(hostname, self.name)
            self._hostnames.append(hostname)
            Inventory.save(inventory)

    def remove_host(self, hostname):
        # type: (str) -> None
        """Remove host from group

        :param hostname: name of the host to remove from the group
        :type hostname: string

        """
        check_arg(hostname, u._('Host name'), str)
        hostname = safe_decode(hostname)
        inventory = Inventory.load()
        inventory.validate_hostnames([hostname])

        group = inventory.get_group(self.name)
        self._hostnames = group.get_hostnames()
        if hostname in self._hostnames:
            # host is associated with group, remove it
            inventory.remove_host(hostname, self.name)
            self._hostnames.remove(hostname)
            Inventory.save(inventory)



# Copyright(c) 2017, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from copy import copy

from kolla_cli.api.exceptions import InvalidArgument
from kolla_cli.api.job import Job
from kolla_cli.common.ansible.actions import KollaAction
from kolla_cli.common.inventory import Inventory
from kolla_cli.common.utils import check_arg
from kolla_cli.common.utils import safe_decode
import kolla_cli.i18n as u


MYPY = False
if MYPY:
    from typing import Dict  # noqa
    from typing import List  # noqa


class HostApi(object):

    @staticmethod
    def host_add(hostnames):
        # type: (List[str]) -> None
        """Add hosts to the inventory

        :param hostnames: list of strings
        """
        check_arg(hostnames, u._('Host names'), list)
        hostnames = safe_decode(hostnames)

        inventory = Inventory.load()
        any_changed = False
        for hostname in hostnames:
            changed = inventory.add_host(hostname)
            if changed:
                any_changed = True
        if any_changed:
            Inventory.save(inventory)

    @staticmethod
    def host_remove(hostnames):
        # type: (List[str]) -> None
        """Remove hosts from the inventory

        :param hostnames: list of strings
        """
        check_arg(hostnames, u._('Host names'), list)
        hostnames = safe_decode(hostnames)

        inventory = Inventory.load()
        any_changed = False
        for hostname in hostnames:
            changed = inventory.remove_host(hostname)
            if changed:
                any_changed = True
        if any_changed:
            Inventory.save(inventory)

    @staticmethod
    def host_get_all():
        # type: () -> List[Host]
        """Get all hosts in the inventory

        :return: Hosts
        :rtype: list of Host objects
        """
        inventory = Inventory.load()
        hosts = []
        host_groups = inventory.get_host_groups()
        for hostname, groupnames in host_groups.items():
            hosts.append(Host(hostname, groupnames))
        return hosts

    @staticmethod
    def host_get(hostnames):
        # type: (List[str]) -> List[Host]
        """Get selected hosts in the inventory

        :param hostnames: list of strings
        :return: hosts
        :rtype: list of Host objects
        """
        check_arg(hostnames, u._('Host names'), list)
        hostnames = safe_decode(hostnames)
        inventory = Inventory.load()
        inventory.validate_hostnames(hostnames)

        hosts = []
        host_groups = inventory.get_host_groups()
        for hostname in hostnames:
            hosts.append(Host(hostname, host_groups[hostname]))
        return hosts

    @staticmethod
    def host_ssh_check(hostnames):
        # type: (List[str]) -> Dict[str,Dict[str,object]]
        """Check hosts for ssh connectivity

        Check status is a dictionary of form:
            - {hostname: {
              'success':<True|False>,
              'msg':message_string},
              ...
              }

        :param hostnames: list of strings
        :return: check status
        :rtype: dictionary
        """
        check_arg(hostnames, u._('Host names'), list)
        inventory = Inventory.load()
        hostnames = safe_decode(hostnames)
        inventory.validate_hostnames(hostnames)
        summary = inventory.ssh_check_hosts(hostnames)
        return summary

    @staticmethod
    def host_setup(hosts_info):
        # type: (Dict[str,Dict[str,object]]) -> None
        """Setup multiple hosts for ssh access

        hosts_info is a dictionary of form:
            - {hostname': {
              'password': password
              'uname': user_name},
              ...
              }

        The uname entry is optional.

        :param hosts_info: dictionary
        """
        check_arg(hosts_info, u._('Hosts info'), dict)
        inventory = Inventory.load()
        inventory.validate_hostnames(hosts_info.keys())
        inventory.setup_hosts(hosts_info)

    @staticmethod
    def host_destroy(hostnames, destroy_type, verbose_level=1,
                     include_data=False, remove_images=False):
        # type: (List[str], str, int, bool, bool) -> Job
        """Destroy Hosts.

        Stops and removes all kolla related docker containers on the
        specified hosts.

        :param hostnames: host names
        :type hostnames: list
        :param destroy_type: either 'kill' or 'stop'
        :type destroy_type: string
        :param verbose_level: the higher the number, the more verbose
        :type verbose_level: integer
        :param include_data: if true, destroy data containers too.
        :type include_data: boolean
        :param remove_images: if true, destroy will remove the docker images
        :type remove_images: boolean
        :return: Job object
        :rtype: Job
        """
        check_arg(hostnames, u._('Host names'), list)
        check_arg(destroy_type, u._('Destroy type'), str)
        check_arg(verbose_level, u._('Verbose level'), int)
        check_arg(include_data, u._('Include data'), bool)
        check_arg(remove_images, u._('Remove images'), bool)
        if destroy_type not in ['stop', 'kill']:
            raise InvalidArgument(
                u._('Invalid destroy type ({type}). Must be either '
                    '"stop" or "kill".').format(type=destroy_type))

        hostnames = safe_decode(hostnames)
        inventory = Inventory.load()
        inventory.validate_hostnames(hostnames)

        action = KollaAction(verbose_level=verbose_level,
                             playbook_name='destroy.yml')
        ansible_job = action.destroy_hosts(hostnames, destroy_type,
                                           include_data, remove_images)
        return Job(ansible_job)


class Host(object):
    """Host"""

    def __init__(self, hostname, groupnames=[]):
        # type: (str, List[str]) -> None
        self.name = hostname
        self._groupnames = groupnames

    def get_name(self):
        # type: () -> str
        """Get name

        :return: host name
        :rtype: string
        """
        return self.name

    def get_groups(self):
        # type: () -> List[str]
        """Get names of the groups associated with this host

        :return: group names
        :rtype: list of strings

        Note: If the groups associated with this host change after this
        host is fetched, the host must be re-fetched to reflect those
        changes.
        """
        return copy(self._groupnames)



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import yaml

from kolla_cli.api.exceptions import InvalidArgument
from kolla_cli.common.properties import AnsibleProperties
from kolla_cli.common.utils import check_arg
from kolla_cli.common.utils import safe_decode
import kolla_cli.i18n as u


MYPY = False
if MYPY:
    from typing import Dict  # noqa
    from typing import List  # noqa

LOG = logging.getLogger(__name__)

GLOBAL_TYPE = 'global'
GROUP_TYPE = 'group'
HOST_TYPE = 'host'
PROP_TYPES = [GLOBAL_TYPE, GROUP_TYPE, HOST_TYPE]


class PropertyApi(object):

    def property_set(self, property_dict,
                     property_type=GLOBAL_TYPE, change_set=None):
        # type: (Dict[str,str], str, List[str]) -> None
        """Set a property

        :param property_dict: property dictionary containing key / values
        :type property_dict: dictionary
        :param property_type: one of 'global', 'group' or 'host'
        :type property_type: string
        :param change_set: for group or host sets this is the list of groups
                           or hosts to set the property for
        :type change_set: list of strings

        """
        ansible_properties = AnsibleProperties()
        for key, value in property_dict.items():
            check_arg(key, u._('Property Key'), str)
            current_property = ansible_properties.get_property(key)
            if current_property is not None:
                current_property_type = current_property.value_type
                if current_property_type is not str:
                    original_value = value
                    value = yaml.safe_load(value)

                    # this check is to make sure that we can assign an empty
                    # string to a property.  without this safe_load will turn
                    # an empty string into a None which is different than an
                    # empty string.
                    if isinstance(original_value, str) and value is None:
                        value = ''
                    if current_property.value is None:
                        current_property_type = None
                    check_arg(value, u._('Property Value'),
                              current_property_type, empty_ok=True)
                    property_dict[key] = value
            else:
                check_arg(value, u._('Property Value'), str, empty_ok=True)
            if type(value) is str and '"' in value:
                raise InvalidArgument(u._('Cannot use double quotes in '
                                          'a property value.'))

        self._check_type(property_type)
        if property_type is not GLOBAL_TYPE:
            check_arg(change_set, u._('Change Set'), list, none_ok=True)
            change_set = safe_decode(change_set)

        if property_type == GLOBAL_TYPE:
            ansible_properties.set_property(property_dict)
        elif property_type == GROUP_TYPE:
            ansible_properties.set_group_property(property_dict, change_set)
        else:
            ansible_properties.set_host_property(property_dict, change_set)

    def property_clear(self, property_list, property_type=GLOBAL_TYPE,
                       change_set=None):
        # type: (List[str], str, List[str]) -> None
        """Clear a property

        :param property_list: property list
        :type property_list: list
        :param property_type: one of 'global', 'group' or 'host'
        :type property_type: string
        :param change_set: for group or host clears this is the list of
                           groups or hosts to clear the property for
        :type change_set: list of strings

        """
        check_arg(property_list, u._('Property List'), list)
        property_list = safe_decode(property_list)

        self._check_type(property_type)
        if property_type is not GLOBAL_TYPE:
            check_arg(change_set, u._('Change Set'), list, none_ok=True)
            change_set = safe_decode(change_set)

        ansible_properties = AnsibleProperties()

        if property_type == GLOBAL_TYPE:
            ansible_properties.clear_property(property_list)
        elif property_type == GROUP_TYPE:
            ansible_properties.clear_group_property(property_list, change_set)
        else:
            ansible_properties.clear_host_property(property_list, change_set)

    def property_get(self, property_type=GLOBAL_TYPE, get_set=None):
        # type: (str, List[str]) -> List[Property]
        """Returns a list of Property objects

        :param property_type: one of 'global', 'group', or 'host'
        :type property_type: string
        :param get_set: optional list of hosts or groups to be used when
                         getting group or host related property lists
        :type get_set: list of strings
        :return: properties
        :rtype: list of Property objects
        """
        self._check_type(property_type)
        get_set = safe_decode(get_set)

        ansible_properties = AnsibleProperties()

        result_list = []
        if property_type == GLOBAL_TYPE:
            property_list = ansible_properties.get_all_unique()
        elif property_type == GROUP_TYPE:
            property_list = ansible_properties.get_group_list(get_set)
        else:
            property_list = ansible_properties.get_host_list(get_set)

        override_flags = ansible_properties.get_all_override_flags()

        for prop in property_list:
            result = Property(prop, override_flags.get(prop.name, None))
            result_list.append(result)

        return result_list

    def _check_type(self, property_type):
        if property_type is None or property_type not in PROP_TYPES:
            raise InvalidArgument(u._('Property Type ({value} is not one of '
                                      'global, group or host')
                                  .format(value=property_type))


class Property(object):
    """Property

    Members:
        - name (str): key
        - value (Any): value
        - file_name (str): name of file property is from
        - overrides (bool): does the property override some other value
        - orig_value (str): the value which is overridden or None
        - target (str): group or host name for group or host properties
        - prop_type (str): one of 'global', 'group' or 'host'
        - ovr_global (bool): true if property is overridden at global level
        - ovr_group (bool): true if property is overridden at group level
        - ovr_host (bool): true if property is overridden at host level
        - value_type (type): the python type of the value
    """

    def __init__(self, ansible_property, override_flags):
        self.name = ansible_property.name
        self.value = ansible_property.value
        self.file_name = ansible_property.file_name
        self.overrides = ansible_property.overrides
        self.orig_value = ansible_property.orig_value
        self.target = ansible_property.target
        self.prop_type = ansible_property.prop_type
        self.value_type = ansible_property.value_type

        if override_flags is not None:
            self.ovr_global = override_flags.ovr_global
            self.ovr_group = override_flags.ovr_group
            self.ovr_host = override_flags.ovr_host
        else:
            self.ovr_global = False
            self.ovr_group = False
            self.ovr_host = False



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from kolla_cli.tests.functional.common import KollaCliTest

import os
import shutil
import tarfile
import unittest

from kolla_cli.api.client import ClientApi
from kolla_cli.common.utils import get_tools_path
from kolla_cli.common.utils import safe_decode

LOGS_PREFIX = '/tmp/kolla_support_logs_'
CLIENT = ClientApi()

LOGDIR = '/tmp/utest_kolla_logs'


class TestFunctional(KollaCliTest):

    def test_log_collector(self):
        hostnames = ['test_host1']
        CLIENT.host_add(hostnames)

        zip_path = ''
        try:
            path = os.path.join(get_tools_path(),
                                'log_collector.py')

            # run the log_collector tool
            retval, msg = self.run_command('/usr/bin/python %s %s'
                                           % (path, 'all'))

            # no host, this should fail
            self.assertIn('error', msg.lower())
        except Exception as e:
            raise e
        finally:
            if zip_path and os.path.exists(zip_path):
                os.remove(zip_path)

    def test_log_collector_api(self):
        if os.path.exists(LOGDIR):
            shutil.rmtree(LOGDIR)
        os.mkdir(LOGDIR)

        hostnames = ['test_host1']
        CLIENT.host_add(hostnames)

        services = CLIENT.service_get_all()
        service_names = []
        for service in services:
            service_names.append(service.name)
        try:
            for hostname in hostnames:
                CLIENT.support_get_logs(service_names, safe_decode(hostname),
                                        LOGDIR)
                raise Exception('get_logs command succeeded without physical '
                                'hosts')
        except Exception as e:
            self.assertIn('UNREACHABLE', str(e),
                          'unexpected failure in get_logs: %s' % str(e))
        finally:
            if os.path.exists(LOGDIR):
                shutil.rmtree(LOGDIR)

    def test_dump(self):
        check_files = [
            'kolla/etc/kolla-cli/ansible/inventory.json',
            'kolla/share/ansible/site.yml',
        ]
        # dump success output is:
        # dump successful to /tmp/kollacli_dump_Umxu6d.tgz
        dump_path = None
        try:
            msg = self.run_cli_cmd('dump')
            self.assertIn('/', msg, 'path not found in dump output: %s' % msg)

            dump_path = '/' + msg.strip().split('/', 1)[1]
            is_file = os.path.isfile(dump_path)
            self.assertTrue(is_file,
                            'dump file not found at %s' % dump_path)
            with tarfile.open(dump_path, 'r') as tar:
                file_paths = tar.getnames()

            for check_file in check_files:
                self.assertIn(check_file, file_paths,
                              'dump: check file: %s, not in files:\n%s'
                              % (check_file, file_paths))
        except Exception as e:
            raise e
        finally:
            if dump_path and os.path.exists(dump_path):
                os.remove(dump_path)


if __name__ == '__main__':
    unittest.main()



# Copyright(c) 2019, caoyuan. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from kolla_cli.common.inventory import Inventory


def check_kolla_args(hostnames=[], servicenames=[]):

    if not any([hostnames, servicenames]):
        return

    inventory = Inventory.load()
    if hostnames:
        inventory.validate_hostnames(hostnames)

    if servicenames:
        inventory.validate_servicenames(servicenames)



#   Copyright 2012-2013 OpenStack Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import oslo_i18n

_translators = oslo_i18n.TranslatorFactory(domain='kolla-cli')

# The primary translation function using the well-known name "_"
_ = _translators.primary

# Translators for log levels.
#
# The abbreviated names are meant to reflect the usual use of a short
# name like '_'. The "L" is for "log" and the other letter comes from
# the level.
_LI = _translators.log_info
_LW = _translators.log_warning
_LE = _translators.log_error
_LC = _translators.log_critical



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from kolla_cli.common.passwords import clear_password
from kolla_cli.common.passwords import get_password_names
from kolla_cli.common.passwords import init_passwords
from kolla_cli.common.passwords import set_password
from kolla_cli.common.passwords import set_password_sshkey
from kolla_cli.common.utils import check_arg
from kolla_cli.common.utils import disallow_chars
import kolla_cli.i18n as u


MYPY = False
if MYPY:
    from typing import List  # noqa


class PasswordApi(object):

    def password_set(self, name, value):
        # type: (str, str) -> None
        """Set password

        :param name: name of the password
        :type name: string
        :param value: value of the password
        :type value: string
        """
        password_name_string = u._('Password name')
        password_value_string = u._('Password value')
        check_arg(name, password_name_string, str)
        disallow_chars(name, password_name_string, '\'')
        check_arg(value, password_value_string, str, display_param=False,
                  empty_ok=True, none_ok=True)
        disallow_chars(value, password_value_string, '\'')
        set_password(name, value)

    def password_set_sshkey(self, name, private_key, public_key):
        # type: (str, str, str) -> None
        """Set password to an ssh key

        :param name: name of the password
        :type name: string
        :param private_key: ssh private key
        :type value: string
        :param public_key: ssh public key
        :type value: string
        """
        password_name_string = u._('Password name')
        private_key_string = u._('Private key')
        public_key_string = u._('Public key')
        check_arg(name, password_name_string, str)
        disallow_chars(name, password_name_string, '\'')
        check_arg(private_key, private_key_string, str, display_param=False)
        disallow_chars(private_key, private_key_string, '\'')
        check_arg(public_key, public_key_string, str, display_param=False)
        disallow_chars(public_key, public_key_string, '\'')
        set_password_sshkey(name, private_key, public_key)

    def password_clear(self, name):
        # type: (str) -> None
        """Clear password

        :param name: name of the password
        :type name: string
        """
        password_name_string = u._('Password name')
        check_arg(name, password_name_string, str)
        disallow_chars(name, password_name_string, '\'')
        clear_password(name)

    def password_get_names(self):
        # type: () -> List[str]
        """Get password names

        :return: password names
        :rtype: list of strings
        """
        return get_password_names()

    def password_init(self):
        # type: () -> None
        """Init empty passwords

        Init empty passwords and ssh keys in /etc/kolla/passwords.yml
        to auto-generated values
        """
        return init_passwords()



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import logging
import os
import tarfile
import tempfile

from kolla_cli.api.exceptions import FailedOperation
from kolla_cli.common.inventory import Inventory
from kolla_cli.common.properties import AnsibleProperties
from kolla_cli.common.utils import get_kolla_ansible_home
from kolla_cli.common.utils import get_kolla_cli_etc
from kolla_cli.common.utils import run_cmd

LOG = logging.getLogger(__name__)


class HostLogs(object):

    def __init__(self, hostname, inventory, servicenames):
        self.hostname = hostname
        self.inventory = inventory
        self.servicenames = servicenames
        self.container_info = {}  # container_id: container_img_name
        self.filtered_servicenames = {}

    def load_container_info(self):
        """get the list of containers on the host"""
        hostname = self.hostname
        err_msg, output = \
            self.inventory.run_ansible_command('-a "docker ps -a"', hostname)
        if err_msg:
            msg = 'Error accessing host %s : %s %s' % (hostname, err_msg,
                                                       output)
            raise FailedOperation(msg)

        if not output:
            msg = ('Host %s is not accessible.' % hostname)
            raise FailedOperation(msg)
        else:
            if '>>' not in output:
                msg = ('Host: %s. Invalid ansible return data: [%s].'
                       % (hostname, output))
                raise FailedOperation(msg)

        if 'NAMES' not in output:
            msg = ('Host: %s. Invalid docker ps return data: [%s].'
                   % (hostname, output))
            raise FailedOperation(msg)

        ansible_properties = AnsibleProperties()
        base_distro = \
            ansible_properties.get_property_value('kolla_base_distro')
        install_type = \
            ansible_properties.get_property_value('kolla_install_type')
        # typically this prefix will be "ol-openstack-"
        container_prefix = base_distro + '-' + install_type + '-'

        # process ps output
        containers = {}

        # the ps output is after the '>>'
        output = output.split('>>', 1)[1]
        LOG.info('docker ps -a on host: %s:\n%s' % (hostname, output))

        lines = output.split('\n')
        for line in lines:
            tokens = line.split()
            if len(tokens) < 2:
                continue
            cid = tokens[0]
            image = tokens[1]
            if container_prefix not in image:
                # skip non-kolla containers
                continue
            name = image.split(container_prefix)[1]
            name = name.split(':')[0]
            containers[cid] = name
        self.container_info = containers

    def get_log(self, container_id):
        """read the container log"""
        hostname = self.hostname
        cmd = '-a "docker logs %s"' % container_id
        err_msg, output = self.inventory.run_ansible_command(cmd, hostname)
        if err_msg:
            msg = 'Error accessing host %s : %s ' % (hostname, err_msg)
            raise FailedOperation(msg)

        if not output:
            msg = ('Host %s is not accessible.' % hostname)
            raise FailedOperation(msg)
        if '>>' not in output:
            msg = ('Host: %s. Invalid ansible return data: [%s].'
                   % (hostname, output))
            raise FailedOperation(msg)

        # the log info is after the '>>'
        output = output.split('>>', 1)[1]
        return output

    def write_logs(self, dirname):
        """write out the log files for all containers"""
        for container_id, container_name in self.filtered_services.items():
            logdata = self.get_log(container_id)
            if logdata:
                logname = '%s_%s.log' % (container_name, container_id)
                self.write_logfile(dirname, logname, logdata)
            else:
                LOG.warn('No log data found for service %s on host %s'
                         % (container_name, self.hostname))

    def write_logfile(self, dirpath, logname, logdata):
        """write out one log file"""
        hostdir = os.path.join(dirpath, self.hostname)
        if not os.path.exists(hostdir):
            os.mkdir(hostdir)
        fpath = os.path.join(hostdir, logname)
        with open(fpath, 'w') as logfile:
            logfile.write(logdata)

    def filter_services(self):
        """filter services to only those of interest"""
        services_subset = {}
        for host_svcid, host_svcname in self.container_info.items():
            for servicename in self.servicenames:
                if (host_svcname == servicename or
                        host_svcname.startswith(servicename + '-')):
                    services_subset[host_svcid] = host_svcname
        self.filtered_services = services_subset


def get_logs(servicenames, hostname, dirname):
    inventory = Inventory.load()
    inventory.validate_hostnames([hostname])
    inventory.validate_servicenames(servicenames, client_filter=True)

    logs = HostLogs(hostname, inventory, servicenames)
    logs.load_container_info()
    logs.filter_services()
    logs.write_logs(dirname)


def dump(dirpath):
    """Dumps configuration data for debugging

    Dumps most files in /etc/kolla and /usr/share/kolla into a
    tar file so be given to support / development to help with
    debugging problems.
    """
    kolla_home = get_kolla_ansible_home()
    kolla_ansible = os.path.join(kolla_home, 'ansible')
    kollacli_etc = get_kolla_cli_etc().rstrip('/')
    ketc = 'kolla/etc/'
    kshare = 'kolla/share/'
    fd, dump_path = tempfile.mkstemp(dir=dirpath, prefix='kollacli_dump_',
                                     suffix='.tgz')
    os.close(fd)  # avoid fd leak
    with tarfile.open(dump_path, 'w:gz') as tar:
        # Can't blanket add kolla_home because the .ssh dir is
        # accessible by the kolla user only (not kolla group)
        tar.add(kolla_ansible,
                arcname=kshare + os.path.basename(kolla_ansible))

        # Can't blanket add kolla_etc because the passwords.yml
        # file is accessible by the kolla user only (not kolla group)
        tar.add(kollacli_etc,
                arcname=ketc + os.path.basename(kollacli_etc))

        # add output of various commands
        _add_cmd_info(tar)

    return dump_path


def _add_cmd_info(tar):
    # run all the kollacli list commands
    cmds = ['kolla-cli --version',
            'kolla-cli service listgroups',
            'kolla-cli service list',
            'kolla-cli group listservices',
            'kolla-cli group listhosts',
            'kolla-cli host list',
            'kolla-cli property list',
            'kolla-cli password list']

    # collect the json inventory output
    inventory = Inventory.load()
    inv_path = inventory.create_json_gen_file()
    cmds.append(inv_path)

    try:
        fd, path = tempfile.mkstemp(suffix='.tmp')
        os.close(fd)
        with open(path, 'w') as tmp_file:
            for cmd in cmds:
                err_msg, output = run_cmd(cmd, False)
                tmp_file.write('\n\n$ %s\n' % cmd)
                if err_msg:
                    tmp_file.write('Error message: %s\n' % err_msg)
                for line in output.split('\n'):
                    tmp_file.write(line + '\n')

        tar.add(path, arcname=os.path.join('kolla', 'cmds_output'))
    except Exception as e:
        raise e
    finally:
        if path:
            os.remove(path)
        inventory.remove_json_gen_file(inv_path)
    return



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from kolla_cli.tests.functional.common import KollaCliTest

import json
import unittest

from kolla_cli.api.client import ClientApi
from kolla_cli.common.ansible_inventory import AnsibleInventory
from kolla_cli.common.inventory import Inventory

CLIENT = ClientApi()


class TestFunctional(KollaCliTest):

    def test_service_api(self):
        service1 = 'nova'
        service2 = 'nova-api'
        exp_services = sorted([service1, service1, service2])
        servicenames = []
        services = CLIENT.service_get(exp_services)
        for service in services:
            servicenames.append(service.name)
        servicenames = sorted(servicenames)
        self.assertEqual(exp_services, servicenames, 'services mis-match')

    def test_service_list(self):
        """$ kolla-cli service list -f json

        [{"Service": "barbican", "Children": ["barbican-keystone-listener"
        "barbican-worker", "barbican-api"]}, {"Service": "barbican-api"
        "Children": []}, {"Service": "barbican-keystone-listener",
        "Children": []}, {"Service": "barbican-worker"...]
        """
        msg = self.run_cli_cmd('service list -f json')
        cli_services = json.loads(msg)
        ansible_inventory = AnsibleInventory()
        ansible_inventory_services = []
        ansible_inventory_service_names = []
        for service in ansible_inventory.services.values():
            if service.is_supported():
                ansible_inventory_services.append(service)
                ansible_inventory_service_names.append(service.name)
        num_services = len(ansible_inventory_services)
        self.assertEqual(num_services, len(cli_services),
                         '# of cli services != expected services.' +
                         '\n\nexpected services: %s'
                         % ansible_inventory_service_names +
                         '\n\ncli services: %s' % cli_services)

    def test_listgroups(self):
        """$ kolla-cli service listgroups

        +------------------------+-------------------------+ \
        | Service                | Groups                  | \
        +------------------------+-------------------------+ \
        | cinder                 | ['control', 'control2'] | \
        | cinder-api             |                         | \
        | cinder-backup          | ['storage']             | \
        | cinder-scheduler       |                         | \
        | cinder-volume          | ['storage']             | \
        | glance                 | ['control', 'control2'] | \
        | glance-api             |                         | \
        | glance-registry        |                         | \
        ...

        """
        msg = self.run_cli_cmd('service listgroups -f json')
        cli_services = json.loads(msg)

        ansible_inventory = AnsibleInventory()
        ansible_inventory_services = []
        ansible_inventory_service_names = []
        for service in ansible_inventory.services.values():
            if service.is_supported():
                ansible_inventory_services.append(service)
                ansible_inventory_service_names.append(service.name)
        num_services = len(ansible_inventory_services)
        self.assertEqual(num_services, len(cli_services),
                         '# of cli services (%s) ' % len(cli_services) +
                         '!= # of expected services (%s).' % num_services +
                         '\n\ncli services: %s' % cli_services)

    def test_service_add_group(self):
        servicename = 'cinder'
        new_group = 'network'

        inventory = Inventory.load()
        service = inventory.get_service(servicename)
        groupnames = service.get_groupnames()

        # add new group to a service
        self.run_cli_cmd('service addgroup %s %s' % (servicename, new_group))

        msg = self.run_cli_cmd('service listgroups -f json')
        cli_services = json.loads(msg)
        cli_service = ''
        for svc in cli_services:
            if svc['Service'] == servicename:
                cli_service = svc
                break
        self.assertNotEqual(cli_service, '',
                            'service: %s, ' % servicename +
                            'not found in cli_services: \n%s'
                            % cli_service)
        cli_groups = cli_service['Groups']
        groupnames.append(new_group)
        self.assertEqual(groupnames, cli_groups,
                         'service: %s, ' % service +
                         'expected groups: %s, ' % groupnames +
                         'cli_groups: %s' % cli_groups)

        # remove that group
        self.run_cli_cmd('service removegroup %s %s' % (servicename,
                                                        new_group))
        groupnames.remove(new_group)

        msg = self.run_cli_cmd('service listgroups -f json')
        cli_services = json.loads(msg)
        for svc in cli_services:
            if svc['Service'] == servicename:
                cli_service = svc
                break
        self.assertNotEqual(cli_service, '',
                            'service: %s, ' % servicename +
                            'not found in cli_services: \n%s'
                            % cli_service)
        cli_groups = cli_service['Groups']
        expected_groups = groupnames
        self.assertEqual(expected_groups, cli_groups,
                         'service: %s, ' % service +
                         'expected groups: %s, ' % expected_groups +
                         'cli_groups: %s' % cli_groups)

        # add new group to a service which has a parent
        servicename = 'glance-api'
        new_group = 'control'
        self.run_cli_cmd('service addgroup %s %s' % (servicename, new_group))

        msg = self.run_cli_cmd('service listgroups -f json')
        cli_services = json.loads(msg)
        for svc in cli_services:
            if svc['Service'] == servicename:
                cli_service = svc
                break
        self.assertNotEqual(cli_service, '',
                            'service: %s, ' % servicename +
                            'not found in cli_services: \n%s'
                            % cli_service)
        cli_groups = cli_service['Groups']
        expected_groups = ['%s' % new_group]
        self.assertEqual(expected_groups, cli_groups,
                         'service: %s, ' % service +
                         'expected groups: %s, ' % expected_groups +
                         'cli_groups: %s' % cli_groups)

        # remove that group
        self.run_cli_cmd('service removegroup %s %s' % (servicename,
                                                        new_group))

        msg = self.run_cli_cmd('service listgroups -f json')
        cli_services = json.loads(msg)
        for svc in cli_services:
            if svc['Service'] == servicename:
                cli_service = svc
                break
        self.assertNotEqual(cli_service, '',
                            'service: %s, ' % service +
                            'not found in cli_services: \n%s'
                            % cli_service)
        cli_groups = cli_service['Groups']
        expected_groups = []
        self.assertEqual(expected_groups, cli_groups,
                         'service: %s, ' % servicename +
                         'expected groups: %s, ' % expected_groups +
                         'cli_groups: %s' % cli_groups)

        test_group = 'testgroup'
        self.run_cli_cmd('group add %s' % test_group)
        self.run_cli_cmd('service addgroup cinder %s' % test_group)
        self.run_cli_cmd('group remove %s' % test_group)
        msg = self.run_cli_cmd('service listgroups -f json')
        self.assertNotIn(test_group, msg,
                         'Group: %s, still listed in services: %s'
                         % (test_group, msg))


if __name__ == '__main__':
    unittest.main()



# Copyright(c) 2017, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from kolla_cli.api.job import Job
from kolla_cli.common.ansible.actions import KollaAction
from kolla_cli.common.ansible.utils import check_kolla_args
from kolla_cli.common.inventory import Inventory
from kolla_cli.common.utils import check_arg
from kolla_cli.common.utils import safe_decode
import kolla_cli.i18n as u

MYPY = False
if MYPY:
    from typing import List  # noqa


class ControlPlaneApi(object):

    @staticmethod
    def deploy(hostnames=[],
               serial_flag=False, verbose_level=1, servicenames=[]):
        # type: (List[str], bool, int, List[str]) -> Job
        """Deploy.

        Deploy and start all kolla containers.

        :param hostnames: hosts to deploy to. If empty, then deploy to all.
        :type hostnames: list of strings
        :param serial_flag: if true, deploy will be done one host at a time
        :type serial_flag: boolean
        :param verbose_level: the higher the number, the more verbose
        :type verbose_level: integer
        :param servicenames: services to deploy. If empty, then deploy all.
        :type servicenames: list of strings
        :return: Job object
        :rtype: Job
        """
        check_arg(hostnames, u._('Host names'), list,
                  empty_ok=True, none_ok=True)
        check_arg(serial_flag, u._('Serial flag'), bool)
        check_arg(verbose_level, u._('Verbose level'), int)
        check_arg(servicenames, u._('Service names'), list,
                  empty_ok=True, none_ok=True)

        check_kolla_args(hostnames=hostnames,
                         servicenames=servicenames)

        hostnames = safe_decode(hostnames)
        servicenames = safe_decode(servicenames)
        action = KollaAction(verbose_level=verbose_level,
                             playbook_name='site.yml')
        ansible_job = action.deploy(hostnames, serial_flag, servicenames)
        return Job(ansible_job)

    @staticmethod
    def prechecks(verbose_level=1, hostnames=[], servicenames=[]):
        # type: (int, List[str], List[str]) -> Job
        """Check pre-deployment configuration of hosts.

        Check if host is ready for a new deployment. This will fail if
        any of the hosts are not configured correctly or if they have
        already been deployed to.
        :param hostnames: host names
        :type hostnames: list
        :param verbose_level: the higher the number, the more verbose
        :type verbose_level: integer
        :param servicenames: services to prechecks.
        :type servicenames: list of strings
        :return: Job object
        :rtype: Job
        """
        check_arg(hostnames, u._('Host names'), list,
                  empty_ok=True, none_ok=True)
        check_arg(verbose_level, u._('Verbose level'), int)
        check_arg(servicenames, u._('Service names'), list,
                  empty_ok=True, none_ok=True)

        check_kolla_args(hostnames=hostnames,
                         servicenames=servicenames)

        hostnames = safe_decode(hostnames)
        servicenames = safe_decode(servicenames)
        action = KollaAction(verbose_level=verbose_level,
                             playbook_name='site.yml')
        ansible_job = action.precheck(hostnames, servicenames)
        return Job(ansible_job)

    @staticmethod
    def pull(verbose_level=1, hostnames=[], servicenames=[]):
        """Pull.

        Pull all images for containers (only pulls, no running container).

        :param verbose_level: the higher the number, the more verbose
        :param hostnames: hosts to pull to. If empty, then pull to all.
        :type hostnames: list of strings
        :type verbose_level: integer
        :param servicenames: services to pull. If empty, then pull all.
        :return: Job object
        :rtype: Job
        """
        check_arg(hostnames, u._('Host names'), list,
                  empty_ok=True, none_ok=True)
        check_arg(verbose_level, u._('Verbose level'), int)
        check_arg(servicenames, u._('Service names'), list,
                  empty_ok=True, none_ok=True)

        check_kolla_args(hostnames=hostnames,
                         servicenames=servicenames)

        hostnames = safe_decode(hostnames)
        servicenames = safe_decode(servicenames)
        action = KollaAction(verbose_level=verbose_level,
                             playbook_name='site.yml')
        ansible_job = action.pull(hostnames, servicenames)
        return Job(ansible_job)

    @staticmethod
    def stop(verbose_level=1, hostnames=[], servicenames=[]):
        # type: (int, List[str], List[str]) -> Job
        """Stop Hosts.

        Stops all kolla related docker containers on the specified hosts.

        :param hostnames: host names
        :type hostnames: list
        :param verbose_level: the higher the number, the more verbose
        :type verbose_level: integer
        :param servicenames: services to stop. If empty, then stop all.
        :type servicenames: list of strings
        :return: Job object
        :rtype: Job
        """
        check_arg(verbose_level, u._('Verbose level'), int)
        check_arg(hostnames, u._('Host names'), list,
                  empty_ok=True, none_ok=True)
        check_arg(servicenames, u._('Service names'), list,
                  empty_ok=True, none_ok=True)

        check_kolla_args(hostnames=hostnames,
                         servicenames=servicenames)

        hostnames = safe_decode(hostnames)
        servicenames = safe_decode(servicenames)
        action = KollaAction(verbose_level=verbose_level,
                             playbook_name='site.yml')
        ansible_job = action.stop(hostnames, servicenames)
        return Job(ansible_job)

    @staticmethod
    def upgrade(verbose_level=1, hostnames=[], servicenames=[]):
        # type: (int, List[str], List[str]) -> Job
        """Upgrade.

        Upgrades existing OpenStack Environment.

        :param verbose_level: the higher the number, the more verbose
        :type verbose_level: integer
        :param hostnames: hostnames to upgrade.
        :type hostnames: list of strings.
        :param servicenames: services to upgrade. If empty, then upgrade all.
        :type servicenames: list of strings
        :return: Job object
        :rtype: Job

        Upgrade containers to new version specified by the property
        "openstack_release."
        """
        check_arg(verbose_level, u._('Verbose level'), int)
        check_arg(hostnames, u._('Host names'), list,
                  empty_ok=True, none_ok=True)
        check_arg(servicenames, u._('Service names'), list,
                  empty_ok=True, none_ok=True)

        check_kolla_args(servicenames=servicenames)

        hostnames = safe_decode(hostnames)
        servicenames = safe_decode(servicenames)
        action = KollaAction(verbose_level=verbose_level,
                             playbook_name='site.yml')
        ansible_job = action.upgrade(hostnames, servicenames)
        return Job(ansible_job)

    @staticmethod
    def genconfig(verbose_level=1, hostnames=[], servicenames=[]):
        # type: (int, List[str], List[str]) -> Job
        """Genconfig.

        Generate configuration files for enabled OpenStack services.

        :param verbose_level: the higher the number, the more verbose
        :type verbose_level: integer
        :param servicenames: services to generate. If empty, then generate all.
        :type servicenames: list of strings
        :return: Job object
        :rtype: Job

        Upgrade containers to new version specified by the property
        "openstack_release."
        """
        check_arg(hostnames, u._('Host names'), list,
                  empty_ok=True, none_ok=True)
        check_arg(verbose_level, u._('Verbose level'), int)
        check_arg(servicenames, u._('Service names'), list,
                  empty_ok=True, none_ok=True)

        check_kolla_args(hostnames=hostnames,
                         servicenames=servicenames)

        hostnames = safe_decode(hostnames)
        servicenames = safe_decode(servicenames)
        action = KollaAction(verbose_level=verbose_level,
                             playbook_name='site.yml')
        ansible_job = action.genconfig(hostnames, servicenames)
        return Job(ansible_job)

    @staticmethod
    def check(verbose_level=1, hostnames=[], servicenames=[]):
        # type: (int, List[str], List[str]) -> Job
        """Do post-deployment smoke tests.

        :param hostnames: host names
        :type hostnames: list
        :param verbose_level: the higher the number, the more verbose
        :type verbose_level: integer
        :param servicenames: services to check. If empty, then check all.
        :type servicenames: list of strings
        :return: Job object
        :rtype: Job
        """
        check_arg(verbose_level, u._('Verbose level'), int)
        check_arg(hostnames, u._('Host names'), list,
                  empty_ok=True, none_ok=True)
        check_arg(servicenames, u._('Service names'), list,
                  empty_ok=True, none_ok=True)

        check_kolla_args(servicenames=servicenames)

        hostnames = safe_decode(hostnames)
        servicenames = safe_decode(servicenames)
        action = KollaAction(verbose_level=verbose_level,
                             playbook_name='site.yml')
        ansible_job = action.check(hostnames, servicenames)
        return Job(ansible_job)

    @staticmethod
    def postdeploy(verbose_level=1):
        """Post-Deploy.

        Do post deploy on deploy node.

        :param verbose_level: the higher the number, the more verbose
        :type verbose_level: integer
        :return: Job object
        :rtype: Job
        """
        check_arg(verbose_level, u._('Verbose level'), int)

        action = KollaAction(verbose_level=verbose_level,
                             playbook_name='post-deploy.yml')
        ansible_job = action.postdeploy()
        return Job(ansible_job)

    @staticmethod
    def reconfigure(verbose_level=1, hostnames=[], servicenames=[]):
        # type: (int, List[str], List[str]) -> Job
        """Reconfigure.

        Reconfigure OpenStack service.

        :param hostnames: host names
        :type hostnames: list
        :param verbose_level: the higher the number, the more verbose
        :type verbose_level: integer
        :param servicenames: services to prechecks.
        :type servicenames: list of strings
        :return: Job object
        :rtype: Job
        """
        check_arg(hostnames, u._('Host names'), list,
                  empty_ok=True, none_ok=True)
        check_arg(verbose_level, u._('Verbose level'), int)
        check_arg(servicenames, u._('Service names'), list,
                  empty_ok=True, none_ok=True)

        check_kolla_args(hostnames=hostnames,
                         servicenames=servicenames)

        hostnames = safe_decode(hostnames)
        servicenames = safe_decode(servicenames)

        check_kolla_args(hostnames=hostnames,
                         servicenames=servicenames)

        action = KollaAction(verbose_level=verbose_level,
                             playbook_name='site.yml')
        ansible_job = action.reconfigure(hostnames, servicenames)
        return Job(ansible_job)

    @staticmethod
    def set_deploy_mode(remote_mode):
        # type: (bool) -> None
        """Set deploy mode to either local or remote.

        Local indicates that the openstack deployment will be
        to the local host. Remote means that the deployment is
        executed via ssh.

        NOTE: local mode is not supported and should never be used
        in production environments.

        :param remote_mode: if remote mode is True deployment is done via ssh
        :type remote_mode: bool
        """
        check_arg(remote_mode, u._('Remote mode'), bool)
        inventory = Inventory.load()
        inventory.set_deploy_mode(remote_mode)
        Inventory.save(inventory)

    @staticmethod
    def get_deploy_mode():
        """Get deploy mode from either local or remote.

        Local indicates that the openstack deployment will be
        to the local host. Remote means that the deployment is
        executed via ssh.

        NOTE: local mode is not supported and should never be used
        in production environments.
        """
        inventory = Inventory.load()
        remote_mode = inventory.remote_mode
        deploy_mode = 'remote' if remote_mode else 'local'
        return deploy_mode



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from kolla_cli.tests.functional.common import KollaCliTest

from kolla_cli.api.client import ClientApi
from kolla_cli.common.ansible_inventory import AnsibleInventory
from kolla_cli.common.inventory import Inventory

import json
import unittest

CLIENT = ClientApi()
UNREACHABLE = 'UNREACHABLE!'


class TestFunctional(KollaCliTest):

    def test_json_generator(self):
        self.run_cli_cmd('setdeploy local')

        host1 = 'host_test1'
        self.run_cli_cmd('host add %s' % host1)

        inventory = Inventory.load()

        path = inventory.create_json_gen_file()
        (retval, msg) = self.run_command(path)
        inventory.remove_json_gen_file(path)
        self.assertEqual(0, retval, 'json generator command failed: %s' % msg)

        self.assertNotEqual('', msg, 'json generator returned no data: %s'
                            % msg)

        self.assertIn(host1, msg, '%s not in json_gen output: %s'
                      % (host1, msg))

        ansible_inventory = AnsibleInventory()
        services = ansible_inventory.services
        for servicename, service in services.items():
            self.assertIn(servicename, msg, '%s not in json_gen output: %s'
                          % (servicename, msg))

        # verify that json output is valid. This will throw if invalid json
        try:
            json.loads(msg)
        except Exception:
            self.assertTrue(False, 'invalid json: %s' % msg)
        remote_msg = '"ansible_ssh_user": "root"'
        local_msg = '"ansible_connection": "local"'

        # verify that setdeploy local worked:
        self.assertIn(local_msg, msg, '%s not in local json_gen output: %s'
                      % (local_msg, msg))
        self.assertNotIn(remote_msg, msg, '%s in local json_gen output: %s'
                         % (remote_msg, msg))

        # verify that setdeploy remote worked:
        CLIENT.set_deploy_mode(remote_mode=True)
        inventory = Inventory.load()
        path = inventory.create_json_gen_file()
        (retval, msg) = self.run_command(path)
        inventory.remove_json_gen_file(path)
        self.assertEqual(0, retval, 'json generator command failed: %s' % msg)

        self.assertIn(remote_msg, msg, '%s not in remote json_gen output: %s'
                      % (remote_msg, msg))
        self.assertNotIn(local_msg, msg, '%s in remote json_gen output: %s'
                         % (local_msg, msg))

    def test_json_filtering(self):

        hosts = ['host_test1', 'host_test2', 'host_test3']
        groups = ['control', 'network', 'storage']

        for host in hosts:
            self.run_cli_cmd('host add %s' % host)
            for groupname in groups:
                group = CLIENT.group_get([groupname])[0]
                group.add_host(host)

        inventory = Inventory.load()

        # filter by host- include all hosts
        inv_filter = {'deploy_hosts': hosts}
        path = inventory.create_json_gen_file(inv_filter)
        self.log.info('run command: %s' % path)
        (retval, msg) = self.run_command(path)
        inventory.remove_json_gen_file(path)
        self.assertEqual(0, retval, 'json generator command failed: %s' % msg)

        self.check_json(msg, groups, hosts, groups, hosts)

        # filter by host- to first host
        included_host = hosts[0]
        inv_filter = {'deploy_hosts': [included_host]}
        path = inventory.create_json_gen_file(inv_filter)
        self.log.info('run command: %s' % path)
        (retval, msg) = self.run_command(path)
        inventory.remove_json_gen_file(path)
        self.assertEqual(0, retval, 'json generator command failed: %s' % msg)
        self.check_json(msg, groups, hosts, groups, [included_host])

        # filter by group- include all groups
        inv_filter = {'deploy_groups': groups}
        path = inventory.create_json_gen_file(inv_filter)
        self.log.info('run command: %s' % path)
        (retval, msg) = self.run_command(path)
        inventory.remove_json_gen_file(path)
        self.assertEqual(0, retval, 'json generator command failed: %s' % msg)
        self.check_json(msg, groups, hosts, groups, hosts)

        # filter by group- to first group
        included_group = groups[0]
        inv_filter = {'deploy_groups': [included_group]}
        path = inventory.create_json_gen_file(inv_filter)
        self.log.info('run command: %s' % path)
        (retval, msg) = self.run_command(path)
        inventory.remove_json_gen_file(path)
        self.assertEqual(0, retval, 'json generator command failed: %s' % msg)
        self.check_json(msg, groups, hosts, [included_group], hosts)

    def test_deploy(self):
        # test will start with no hosts in the inventory
        # deploy will throw an exception if it fails
        # disable all services first as without it empty groups cause errors
        enable_service_props = {}
        for service in CLIENT.service_get_all():
            service_name = service.name.replace('-', '_')
            enable_service_props['enable_%s' % service_name] = 'no'
        CLIENT.property_set(enable_service_props)

        msg = ''
        CLIENT.set_deploy_mode(remote_mode=False)
        job = CLIENT.deploy(hostnames=[])
        job.wait()
        msg = job.get_console_output()
        self.assertEqual(job.get_status(), 0,
                         'error performing whole host deploy %s' % msg)

        # test deploy with timeout
        msg = self.run_cli_cmd('action deploy --timeout .001',
                               expect_error=True)
        self.assertIn('timed out', msg)

    def test_upgrade(self):
        # test will upgrade an environment with no hosts, mostly a NOP,
        # but it will go through the client code paths.
        self.run_cli_cmd('action upgrade -v')

        msg = ''
        # run rabbitmq service deploy
        try:
            CLIENT.host_add(['dummy_host'])
            CLIENT.set_deploy_mode(remote_mode=True)
            job = CLIENT.upgrade()
            job.wait()
            msg = job.get_console_output()
            self.assertEqual(job.get_status(), 1,
                             'upgrade succeeded unexpectedly: %s'
                             % msg)
            self.assertIn(UNREACHABLE, msg)
        except Exception as e:
            raise e
        finally:
            CLIENT.host_remove(['dummy_host'])

    def test_pull(self):
        # test will upgrade an environment with no hosts, mostly a NOP,
        # but it will go through the client code paths.
        self.run_cli_cmd('action pull -v')

        CLIENT.host_add(['test_pull_host'])
        CLIENT.set_deploy_mode(remote_mode=True)
        job = CLIENT.pull()
        job.wait()
        msg = job.get_console_output()
        self.assertEqual(job.get_status(), 1,
                         'pull succeeded unexpectedly: %s'
                         % msg)
        self.assertIn(UNREACHABLE, msg)

    def check_json(self, msg, groups, hosts, included_groups, included_hosts):
        err_msg = ('included groups: %s\n' % included_groups +
                   'included hosts: %s\n' % included_hosts)
        inv_dict = json.loads(msg)
        for group in groups:
            group_hosts = inv_dict[group]['hosts']
            for host in hosts:
                if group in included_groups and host in included_hosts:
                    self.assertIn(host, group_hosts, err_msg +
                                  '%s not in %s' % (host, group_hosts))
                else:
                    self.assertNotIn(host, group_hosts, err_msg +
                                     '%s still in %s' % (host, group_hosts))


if __name__ == '__main__':
    unittest.main()



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from copy import copy
from kolla_cli.common.utils import get_admin_user

ANSIBLE_BECOME = 'ansible_become'
ANSIBLE_SSH_USER = 'ansible_ssh_user'
ANSIBLE_CONNECTION = 'ansible_connection'


class HostGroup(object):
    class_version = 1

    def __init__(self, name):
        self.name = name
        self.hostnames = []
        self.vars = {}
        self.version = self.__class__.class_version

    def upgrade(self):
        pass

    def add_host(self, host):
        if host.name is not None and host.name not in self.hostnames:
            self.hostnames.append(host.name)

    def remove_host(self, host):
        if host.name in self.hostnames:
            self.hostnames.remove(host.name)

    def get_hostnames(self):
        return copy(self.hostnames)

    def get_vars(self):
        return self.vars.copy()

    def set_var(self, name, value):
        self.vars[name] = value

    def clear_var(self, name):
        if name in self.vars:
            del self.vars[name]

    def set_remote(self, remote_flag):
        self.set_var(ANSIBLE_BECOME, 'yes')
        if remote_flag:
            # set the ssh info for all the servers in the group
            self.set_var(ANSIBLE_SSH_USER, get_admin_user())
            self.clear_var(ANSIBLE_CONNECTION)
        else:
            # remove ssh info, add local connection type
            self.set_var(ANSIBLE_CONNECTION, 'local')
            self.clear_var(ANSIBLE_SSH_USER)



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from kolla_cli.tests.functional.common import KollaCliTest

from kolla_cli.api.client import ClientApi

import unittest

CLIENT = ClientApi()


class TestFunctional(KollaCliTest):

    def test_reconfigure(self):
        # test will start with no hosts in the inventory
        # reconfigure will throw an exception if it fails
        # disable all services first as without it empty groups cause errors
        enable_service_props = {}
        for service in CLIENT.service_get_all():
            service_name = service.name.replace('-', '_')
            enable_service_props['enable_%s' % service_name] = 'no'
        CLIENT.property_set(enable_service_props)

        msg = ''
        try:
            job = CLIENT.reconfigure()
            job.wait()
            msg = job.get_console_output()
            self.assertEqual(job.get_status(), 0,
                             'error performing reconfigure: %s' % msg)
        except Exception as e:
            self.assertEqual(0, 1,
                             'unexpected exception in reconfigure %s, %s'
                             % (e.message, msg))


if __name__ == '__main__':
    unittest.main()



# Copyright(c) 2017, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import time
import traceback

from cliff.command import Command

from kolla_cli.api.client import ClientApi
from kolla_cli.commands.exceptions import CommandError
from kolla_cli.common.utils import handers_action_result
import kolla_cli.i18n as u

CLIENT = ClientApi()
LOG = logging.getLogger(__name__)


class Deploy(Command):
    """Deploy and start all kolla containers."""
    def get_parser(self, prog_name):
        parser = super(Deploy, self).get_parser(prog_name)
        parser.add_argument('--hosts', nargs='?',
                            metavar='<host_list>',
                            help=u._('Deployment host list'))
        parser.add_argument('--serial', action='store_true',
                            help=u._('Deploy serially'))
        parser.add_argument('--timeout', nargs=1,
                            metavar='<timeout>',
                            help=u._('timeout (in minutes)'))
        parser.add_argument('--services', nargs='?',
                            metavar='<service_list>',
                            help=u._('Deploy service list'))
        return parser

    def take_action(self, parsed_args):
        hosts = None
        serial_flag = False
        verbose_level = self.app.options.verbose_level
        timeout_target = 0
        services = None
        try:
            if parsed_args.hosts:
                host_list = parsed_args.hosts.strip()
                hosts = host_list.split(',')
            if parsed_args.serial:
                serial_flag = True
            if parsed_args.timeout:
                try:
                    timeout = float(parsed_args.timeout[0])
                except Exception:
                    raise CommandError(u._('Timeout value is not a number.'))
                timeout_target = time.time() + (60 * timeout)
            if parsed_args.services:
                service_list = parsed_args.services.strip()
                services = service_list.split(',')

            # if we are doing a targeted host deploy make sure we are doing it
            # to only compute nodes
            if hosts:
                invalid_host_list = []
                compute_group = CLIENT.group_get(['compute'])[0]
                compute_hosts = compute_group.get_hosts()
                for host in hosts:
                    if host not in compute_hosts:
                        invalid_host_list.append(host)
                if len(invalid_host_list) > 0:
                    raise CommandError(
                        u._('Invalid hosts for host targeted deploy. '
                            'Hosts must be in the compute group only.'
                            'Invalid hosts: {hosts}')
                        .format(hosts=invalid_host_list))

            job = CLIENT.deploy(hosts, serial_flag, verbose_level, services)

            # wait for job to complete
            status = None
            while status is None:
                if timeout_target and time.time() > timeout_target:
                    job.kill()
                    raise CommandError(u._('Job timed out and was killed.'))
                time.sleep(1)
                status = job.get_status()

            # job is done
            handers_action_result(job, status, verbose_level)
        except Exception:
            raise Exception(traceback.format_exc())


class Prechecks(Command):
    """Do pre-deployment checks for hosts."""
    def get_parser(self, prog_name):
        parser = super(Prechecks, self).get_parser(prog_name)
        parser.add_argument('--hosts', nargs='?',
                            metavar='<host_list>',
                            help=u._('Precheck host list'))
        parser.add_argument('--services', nargs='?',
                            metavar='<service_list>',
                            help=u._('Precheck service list'))
        return parser

    def take_action(self, parsed_args):
        hosts = []
        services = []
        try:
            verbose_level = self.app.options.verbose_level

            if parsed_args.hosts:
                host_list = parsed_args.hosts.strip()
                hosts = host_list.split(',')
            if parsed_args.services:
                service_list = parsed_args.services.strip()
                services = service_list.split(',')
            job = CLIENT.prechecks(verbose_level, hosts, services)
            status = job.wait()
            handers_action_result(job, status, verbose_level)
        except Exception:
            raise Exception(traceback.format_exc())


class Pull(Command):
    """Pull all images for containers (only pulls, no running container)."""
    def get_parser(self, prog_name):
        parser = super(Pull, self).get_parser(prog_name)
        parser.add_argument('--hosts', nargs='?',
                            metavar='<host_list>',
                            help=u._('Pull host list'))
        parser.add_argument('--services', nargs='?',
                            metavar='<service_list>',
                            help=u._('Pull service list'))
        return parser

    def take_action(self, parsed_args):
        hosts = []
        services = []
        try:
            verbose_level = self.app.options.verbose_level
            if parsed_args.hosts:
                host_list = parsed_args.hosts.strip()
                hosts = host_list.split(',')
            if parsed_args.services:
                service_list = parsed_args.services.strip()
                services = service_list.split(',')
            job = CLIENT.pull(verbose_level, hosts, services)
            status = job.wait()
            handers_action_result(job, status, verbose_level)
        except Exception:
            raise Exception(traceback.format_exc())


class Reconfigure(Command):
    """Reconfigure OpenStack service."""
    def get_parser(self, prog_name):
        parser = super(Reconfigure, self).get_parser(prog_name)
        parser.add_argument('--hosts', nargs='?',
                            metavar='<host_list>',
                            help=u._('Reconfigure host list'))
        parser.add_argument('--services', nargs='?',
                            metavar='<service_list>',
                            help=u._('Reconfigure service list'))
        return parser

    def take_action(self, parsed_args):
        hosts = []
        services = []
        try:
            verbose_level = self.app.options.verbose_level
            if parsed_args.hosts:
                host_list = parsed_args.hosts.strip()
                hosts = host_list.split(',')
            if parsed_args.services:
                service_list = parsed_args.services.strip()
                services = service_list.split(',')
            job = CLIENT.reconfigure(verbose_level, hosts, services)
            status = job.wait()
            handers_action_result(job, status, verbose_level)
        except Exception:
            raise Exception(traceback.format_exc())


class Stop(Command):
    """Stop all kolla containers on host(s).

    Stops all kolla related docker containers on either the
    specified host or all hosts if the hostname all is used.
    """
    def get_parser(self, prog_name):
        parser = super(Stop, self).get_parser(prog_name)
        parser.add_argument('--hosts', nargs='?',
                            metavar='<host_list>',
                            help=u._('Stop host list'))
        parser.add_argument('--services', nargs='?',
                            metavar='<service_list>',
                            help=u._('Stop service list'))
        return parser

    def take_action(self, parsed_args):
        try:
            hosts = []
            services = []
            verbose_level = self.app.options.verbose_level
            if parsed_args.hosts:
                host_list = parsed_args.hosts.strip()
                hosts = host_list.split(',')
            if parsed_args.services:
                service_list = parsed_args.services.strip()
                services = service_list.split(',')
            job = CLIENT.stop(verbose_level, hosts, services)
            status = job.wait()
            handers_action_result(job, status, verbose_level)
        except Exception:
            raise Exception(traceback.format_exc())


class Upgrade(Command):
    """Upgrades existing OpenStack Environment."""
    def get_parser(self, prog_name):
        parser = super(Upgrade, self).get_parser(prog_name)
        parser.add_argument('--hosts', nargs='?',
                            metavar='<host_list>',
                            help=u._('Upgrade host list'))
        parser.add_argument('--services', nargs='?',
                            metavar='<service_list>',
                            help=u._('Upgrade service list'))
        return parser

    def take_action(self, parsed_args):
        hosts = []
        services = []
        try:
            if parsed_args.hosts:
                host_list = parsed_args.hosts.strip()
                hosts = host_list.split(',')
            if parsed_args.services:
                service_list = parsed_args.services.strip()
                services = service_list.split(',')
            verbose_level = self.app.options.verbose_level
            job = CLIENT.upgrade(verbose_level, hosts, services)
            status = job.wait()
            handers_action_result(job, status, verbose_level)
        except Exception:
            raise Exception(traceback.format_exc())


class Check(Command):
    """Do post-deployment smoke tests."""
    def get_parser(self, prog_name):
        parser = super(Check, self).get_parser(prog_name)
        parser.add_argument('--hosts', nargs='?',
                            metavar='<host_list>',
                            help=u._('Check host list'))
        parser.add_argument('--services', nargs='?',
                            metavar='<service_list>',
                            help=u._('Check service list'))
        return parser

    def take_action(self, parsed_args):
        hosts = []
        services = None
        try:
            if parsed_args.hosts:
                host_list = parsed_args.hosts.strip()
                hosts = host_list.split(',')
            if parsed_args.services:
                service_list = parsed_args.services.strip()
                services = service_list.split(',')
            verbose_level = self.app.options.verbose_level
            job = CLIENT.check(verbose_level, hosts, services)
            status = job.wait()
            handers_action_result(job, status, verbose_level)
        except Exception:
            raise Exception(traceback.format_exc())


class Genconfig(Command):
    """Generate configuration files for enabled OpenStack services."""
    def get_parser(self, prog_name):
        parser = super(Genconfig, self).get_parser(prog_name)
        parser.add_argument('--hosts', nargs='?',
                            metavar='<host_list>',
                            help=u._('genarate configs host list'))
        parser.add_argument('--services', nargs='?',
                            metavar='<service_list>',
                            help=u._('genarate configs service list'))
        return parser

    def take_action(self, parsed_args):
        hosts = []
        services = []
        try:
            verbose_level = self.app.options.verbose_level
            if parsed_args.hosts:
                host_list = parsed_args.hosts.strip()
                hosts = host_list.split(',')
            if parsed_args.services:
                service_list = parsed_args.services.strip()
                services = service_list.split(',')
            job = CLIENT.genconfig(verbose_level, hosts, services)
            status = job.wait()
            handers_action_result(job, status, verbose_level)
        except Exception:
            raise Exception(traceback.format_exc())


class PostDeploy(Command):
    """Do post deploy on deploy node."""

    def take_action(self, parsed_args):
        verbose_level = self.app.options.verbose_level
        try:
            job = CLIENT.postdeploy(verbose_level)
            status = job.wait()
            handers_action_result(job, status, verbose_level)
        except Exception:
            raise Exception(traceback.format_exc())


class CertificateInit(Command):
    """Generates self-signed certificate"""

    def take_action(self, parsed_args):
        verbose_level = self.app.options.verbose_level
        try:
            job = CLIENT.certificate_init(verbose_level)

            # wait for job to complete
            status = job.wait()
            handers_action_result(job, status, verbose_level)
        except Exception:
            raise Exception(traceback.format_exc())



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import tempfile
import traceback

from cliff.command import Command

from kolla_cli.api.client import ClientApi
import kolla_cli.i18n as u

LOG = logging.getLogger(__name__)
CLIENT = ClientApi()


class Dump(Command):
    """Dumps configuration data for debugging.

    Dumps most files in /etc/kolla and /usr/share/kolla into a
    tar file so be given to support / development to help with
    debugging problems.
    """
    def take_action(self, parsed_args):
        try:
            dump_path = CLIENT.support_dump(tempfile.gettempdir())
            LOG.info(u._('Dump successful to {path}').format(path=dump_path))
        except Exception:
            msg = (u._('Dump failed: {reason}')
                   .format(reason=traceback.format_exc()))
            raise Exception(msg)



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from kolla_cli.tests.functional.common import KollaCliTest

import json
import os
import unittest
import yaml

from kolla_cli.api.client import ClientApi
from kolla_cli.api.exceptions import NotInInventory
from kolla_cli.api.host import Host

TEST_YML_FNAME = 'unittest_hosts_setup.yml'

CLIENT = ClientApi()


class TestFunctional(KollaCliTest):

    def test_host_add_remove(self):
        hosts = {}

        msg = self.run_cli_cmd('host list -f json')
        self._check_cli_output(hosts, msg)

        host1 = 'host_test1'
        host2 = 'host_test2'

        group1 = 'control'

        hosts[host1] = Host(host1)
        self.run_cli_cmd('host add %s' % host1)
        msg = self.run_cli_cmd('host list -f json')
        self._check_cli_output(hosts, msg)

        hosts[host2] = Host(host2)
        self.run_cli_cmd('host add %s' % host2)
        msg = self.run_cli_cmd('host list -f json')
        self._check_cli_output(hosts, msg)

        del hosts[host2]
        self.run_cli_cmd('host remove %s' % host2)
        msg = self.run_cli_cmd('host list -f json')
        self._check_cli_output(hosts, msg)

        del hosts[host1]
        self.run_cli_cmd('host remove %s' % host1)
        msg = self.run_cli_cmd('host list -f json')
        self._check_cli_output(hosts, msg)

        # check remove all
        hosts[host1] = Host(host1)
        self.run_cli_cmd('host add %s' % host1)
        hosts[host2] = Host(host2)
        self.run_cli_cmd('host add %s' % host2)
        msg = self.run_cli_cmd('host list -f json')
        self._check_cli_output(hosts, msg)

        hosts.clear()
        self.run_cli_cmd('host remove all')
        msg = self.run_cli_cmd('host list -f json')
        self._check_cli_output(hosts, msg)

        # check groups in host list
        hosts[host1] = Host(host1, [group1])
        self.run_cli_cmd('host add %s' % host1)
        self.run_cli_cmd('group addhost %s %s' % (group1, host1))
        msg = self.run_cli_cmd('host list -f json')
        self._check_cli_output(hosts, msg)

        # removing group by resetting host1 to new Host with no groups
        hosts[host1] = Host(host1)
        self.run_cli_cmd('group removehost %s %s' % (group1, host1))
        msg = self.run_cli_cmd('host list -f json')
        self._check_cli_output(hosts, msg)

    def test_host_api(self):
        # check some of the api not exercised by the CLI
        host1 = 'host_test1'
        host2 = 'host_test2'
        exp_hosts = sorted([host1, host2])
        CLIENT.host_add(exp_hosts)
        hosts = CLIENT.host_get([host1])
        hostnames = []
        for host in hosts:
            hostnames.append(host.name)
        self.assertIn(host1, hostnames, 'host %s is missing in %s'
                      % (host1, hostnames))
        self.assertNotIn(host2, hostnames, 'host %s is unexpectedly in %s'
                         % (host2, hostnames))
        hosts = CLIENT.host_get(exp_hosts)
        hostnames = []
        for host in hosts:
            hostnames.append(host.name)
        self.assertEqual(exp_hosts, sorted(hostnames), 'hosts mismatch')

        CLIENT.host_remove(exp_hosts)
        try:
            CLIENT.host_get(exp_hosts)
            self.assertTrue(False, 'Failed to raise NotInInventory exception')
        except NotInInventory:
            pass
        except Exception as e:
            raise e

        # check the type checking logic
        self.check_types(CLIENT.host_add, [list])
        self.check_types(CLIENT.host_remove, [list])
        self.check_types(CLIENT.host_setup, [dict])
        self.check_types(CLIENT.host_ssh_check, [list])
        self.check_types(CLIENT.host_destroy, [list, str, int, bool])

    def test_host_list_nonascii(self):
        hostname = 'host_test1'
        CLIENT.host_add([hostname])

        # this is a groupname in cyrillic chars
        groupname1 = u'\u0414\u0435\u043a\u0430\u0442'
        groupname2 = 'test_group2'  # ascii groupname
        groupnames = [groupname1, groupname2]
        CLIENT.group_add(groupnames)
        groups = CLIENT.group_get(groupnames)
        for group in groups:
            group.add_host(hostname)

# TODO(bmace) -- test currently broken
# msg = self.run_cli_cmd('host list')
# self.assertIn(groupname1, msg)
# self.assertNotIn("u'\u0414\u0435\u043a\u0430\u0442'", msg, 'groupname '
#                  'incorrectly appearing as unicode bytes in output')
# self.assertNotIn("u'test_group2'", msg, 'unicode escape text is '
#                  'incorrectly displayed in host list output')

    def _check_cli_output(self, exp_hosts, cli_output):
        """Verify cli data against model data

        The host list cli output looks like this:

            $ host list -f json
            [{"Host": "foo", "Groups": ["control", "network"]}]
        """
        # check for any host in cli output that shouldn't be there
        cli_hosts = json.loads(cli_output)

        if not exp_hosts:
            if len(cli_hosts) == 1:
                cli_hostname = cli_hosts[0]['Host']
                if not cli_hostname:
                    # both cli and expected hosts are None
                    return

        for cli_host in cli_hosts:
            cli_hostname = cli_host['Host']
            self.assertIn(cli_hostname, exp_hosts,
                          'unexpected host: %s, found in cli output: %s'
                          % (cli_hostname, cli_output))

        # check that all expected hosts are in the output
        for exp_host in exp_hosts.values():
            exp_host_found = False
            for cli_host in cli_hosts:
                if exp_host.get_name() == cli_host['Host']:
                    exp_host_found = True
                    cli_groups = cli_host['Groups']
                    exp_groups = exp_host.get_groups()
                    self.assertEqual(exp_groups, cli_groups)

            self.assertTrue(exp_host_found,
                            'hostname: %s not in cli output: \n%s'
                            % (exp_host.get_name(), cli_output))

    def write_yml(self, yml_dict):
        yml = yaml.dump(yml_dict)
        with open(self.get_yml_path(), 'w') as yml_file:
            yml_file.write(yml)

    def get_yml_path(self):
        home = os.path.expanduser('~')
        return os.path.join(home, TEST_YML_FNAME)


if __name__ == '__main__':
    unittest.main()



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import traceback

from cliff.command import Command
from cliff.lister import Lister

from kolla_cli.api.client import ClientApi
from kolla_cli.commands.exceptions import CommandError
from kolla_cli.common import utils
import kolla_cli.i18n as u

CLIENT = ClientApi()


def _get_names(args_list):
    csv_list = args_list[0].strip()
    names = csv_list.split(',')
    if 'all' in names:
        names = None
    return names


class PropertySet(Command):
    "Property Set"

    def get_parser(self, prog_name):
        parser = super(PropertySet, self).get_parser(prog_name)
        parser.add_argument('propertyname', metavar='<propertyname>',
                            help=u._('Property name'))
        parser.add_argument('propertyvalue', metavar='<propertyvalue',
                            help=u._('Property value'))
        parser.add_argument('--hosts', nargs=1,
                            metavar='<host_list>',
                            help=u._('Property host list'))
        parser.add_argument('--groups', nargs=1,
                            metavar='<group_list>',
                            help=u._('Property group list'))
        return parser

    def take_action(self, parsed_args):
        try:
            property_name = parsed_args.propertyname.strip()
            property_value = parsed_args.propertyvalue.strip()
            property_dict = {}
            property_dict[property_name] = property_value

            if parsed_args.hosts:
                if parsed_args.groups:
                    raise CommandError(
                        u._('Invalid to use both hosts and groups arguments '
                            'together.'))

                host_names = _get_names(parsed_args.hosts)

                CLIENT.property_set(property_dict,
                                    'host', host_names)

            elif parsed_args.groups:
                group_names = _get_names(parsed_args.groups)

                CLIENT.property_set(property_dict,
                                    'group', group_names)
            else:
                CLIENT.property_set(property_dict,
                                    'global')

        except Exception:
            raise Exception(traceback.format_exc())


class PropertyClear(Command):
    "Property Clear"

    def get_parser(self, prog_name):
        parser = super(PropertyClear, self).get_parser(prog_name)
        parser.add_argument('propertyname', metavar='<propertyname>',
                            help=u._('Property name'))
        parser.add_argument('--hosts', nargs=1,
                            metavar='<host_list>',
                            help=u._('Property host list'))
        parser.add_argument('--groups', nargs=1,
                            metavar='<group_list>',
                            help=u._('Property group list'))
        return parser

    def take_action(self, parsed_args):
        try:
            property_name = parsed_args.propertyname.strip()
            property_list = []
            property_list.append(property_name)

            if parsed_args.hosts:
                if parsed_args.groups:
                    raise CommandError(
                        u._('Invalid to use both hosts and groups arguments '
                            'together.'))

                host_names = _get_names(parsed_args.hosts)

                CLIENT.property_clear(property_list, 'host',
                                      host_names)
            elif parsed_args.groups:
                group_names = _get_names(parsed_args.groups)

                CLIENT.property_clear(property_list, 'group',
                                      group_names)
            else:
                CLIENT.property_clear(property_list, 'global')

        except Exception:
            raise Exception(traceback.format_exc())


class PropertyList(Lister):
    """List all properties."""

    def __init__(self, app, app_args, cmd_name=None):
        super(Lister, self).__init__(app, app_args,
                                     cmd_name=cmd_name)

        self.is_global = True
        self.is_all_flag = False
        self.is_long_flag = False
        self.list_type = None

    def get_parser(self, prog_name):
        parser = super(PropertyList, self).get_parser(prog_name)
        parser.add_argument('--all', action='store_true',
                            help=u._('List all properties'))
        parser.add_argument('--long', action='store_true',
                            help=u._('Show all property attributes'))
        parser.add_argument('--hosts', nargs=1,
                            metavar='<host_list>',
                            help=u._('Property host list'))
        parser.add_argument('--groups', nargs=1,
                            metavar='<group_list>',
                            help=u._('Property group list'))
        return parser

    def take_action(self, parsed_args):
        try:
            if parsed_args.all:
                self.is_all_flag = True
            if parsed_args.long:
                self.is_long_flag = True

            if parsed_args.hosts:
                if parsed_args.groups:
                    raise CommandError(
                        u._('Invalid to use both hosts and groups arguments '
                            'together.'))

                self.is_global = False
                self.list_type = u._('Host')
                host_names = _get_names(parsed_args.hosts)

                property_list = CLIENT.property_get('host',
                                                    host_names)

            elif parsed_args.groups:
                self.is_global = False
                self.list_type = u._('Group')
                group_names = _get_names(parsed_args.groups)
                property_list = CLIENT.property_get('group',
                                                    group_names)

            else:
                property_list = CLIENT.property_get('global')

            data = self._get_list_data(property_list)
            header = self._get_list_header()
            return (header, data)

        except Exception:
            raise Exception(traceback.format_exc())

    def _get_list_header(self):
        header = None
        if self.is_long_flag:
            if self.is_global:
                header = (u._('OVR'),
                          u._('Property Name'), u._('Property Value'),
                          u._('Original Value'))
            else:
                header = (u._('OVR'),
                          u._('Property Name'), u._('Property Value'),
                          u._('Original Value'),
                          self.list_type)
        else:
            if self.is_global:
                header = (u._('OVR'),
                          u._('Property Name'), u._('Property Value'))
            else:
                header = (u._('OVR'),
                          u._('Property Name'), u._('Property Value'),
                          self.list_type)
        return header

    def _get_list_data(self, property_list):
        data = []
        if property_list:
            property_length = utils.get_property_list_length()
            for prop in property_list:
                include_prop = False
                if (prop.value is not None and
                        len(str(prop.value)) > property_length):
                    if self.is_all_flag:
                        include_prop = True
                else:
                    include_prop = True

                if not include_prop:
                    continue

                ovr_global = '-'
                ovr_group = '-'
                ovr_host = '-'

                if prop.ovr_global:
                    ovr_global = '*'
                if prop.ovr_group:
                    ovr_group = 'G'
                if prop.ovr_host:
                    ovr_host = 'H'

                prop_ovr = ovr_global + ovr_group + ovr_host

                if self.is_long_flag:
                    if self.is_global:
                        data.append((prop_ovr, prop.name, prop.value,
                                     prop.orig_value))
                    else:
                        data.append((prop_ovr, prop.name, prop.value,
                                     prop.orig_value, prop.target))
                else:
                    if self.is_global:
                        data.append((prop_ovr, prop.name, prop.value))
                    else:
                        data.append((prop_ovr, prop.name, prop.value,
                                     prop.target))
        else:
            if self.is_long_flag:
                if self.is_global:
                    data.append(('', '', '', ''))
                else:
                    data.append(('', '', '', '', ''))
            else:
                if self.is_global:
                    data.append(('', '', ''))
                else:
                    data.append(('', '', '', ''))

        return data



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os.path
import paramiko
import traceback

from kolla_cli.common.utils import get_admin_user
from kolla_cli.common.utils import get_kolla_cli_etc
from kolla_cli.common.utils import get_setup_user
import kolla_cli.i18n as u

MIN_DOCKER_VERSION = '1.10.0'

LOG = logging.getLogger(__name__)


def ssh_connect(net_addr, username, password):
    try:
        ssh_client = paramiko.SSHClient()
        ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh_client.connect(hostname=net_addr, username=username,
                           password=password)
        return ssh_client
    except Exception:
        _close_ssh_client(ssh_client)
        raise Exception(traceback.format_exc())


def ssh_setup_host(net_addr, password, setup_user=None):
    admin_user = get_admin_user()
    if setup_user is None:
        setup_user = get_setup_user()
    public_key = ssh_get_public_key()
    ssh_client = None

    try:
        ssh_client = ssh_connect(net_addr, setup_user, password)
        # populate authorized keys file w/ public key
        key_dir = os.path.join(os.path.expanduser('~%s' % admin_user),
                               '.ssh', 'authorized_keys')
        cmd = ('/usr/bin/sudo su - %s -c "echo \'%s\' >> %s"'
               % (admin_user, public_key, key_dir))
        _exec_ssh_cmd(cmd, ssh_client)
    except Exception as e:
        raise e
    finally:
        _close_ssh_client(ssh_client)


def _close_ssh_client(ssh_client):
    if ssh_client:
        try:
            ssh_client.close()
        except Exception:  # nosec
            pass


def _exec_ssh_cmd(cmd, ssh_client):
    LOG.debug(cmd)
    _, stdout, stderr = ssh_client.exec_command(cmd, get_pty=True)  # nosec
    msg = stdout.read()
    errmsg = stderr.read()
    LOG.debug('%s : %s' % (msg, errmsg))
    if errmsg:
        LOG.warn(
            u._LW('WARNING: command : {command})\nmessage : {message}')
            .format(command=cmd, message=errmsg.strip()))
    return msg, errmsg


def ssh_get_public_key():
    keyfile_path = os.path.join(get_kolla_cli_etc(), 'id_rsa.pub')
    with open(keyfile_path, "r") as public_key_file:
        public_key = public_key_file.read()
        return public_key



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fcntl
import json
import logging
import os
import pwd
import re
import subprocess  # nosec
import time

from kolla_cli.common.inventory import remove_temp_inventory
from kolla_cli.common.utils import get_kolla_actions_path
from kolla_cli.common.utils import Lock
from kolla_cli.common.utils import PidManager
from kolla_cli.common.utils import run_cmd
from kolla_cli.common.utils import safe_decode
import kolla_cli.i18n as u

LOG = logging.getLogger(__name__)

LINE_LENGTH = 80

ANSIBLE_1_OR_MORE = 'One or more items failed'


class AnsibleJob(object):
    """class for running ansible commands"""

    def __init__(self, cmd, deploy_id, print_output, inventory_path):
        self._command = cmd
        self._deploy_id = deploy_id
        self._print_output = print_output
        self._temp_inv_path = inventory_path

        self._process = None
        self._process_std_err = None
        self._errors = []
        self._error_total = 0
        self._ignore_total = 0
        self._cmd_output = ''
        self._kill_uname = None
        self._ansible_lock = Lock(owner='ansible_job')
        self._ignore_error_strings = None
        self._host_ignored_error_count = {}

    def run(self):
        try:
            locked = self._ansible_lock.wait_acquire()
            if not locked:
                raise Exception(
                    u._('unable to get lock: {lock}, to run '
                        'ansible job: {cmd} ')
                    .format(lock=self._ansible_lock.lockpath,
                            cmd=self._command))

            LOG.debug('playbook command: %s' % self._command)
            # ansible 2.2 and later introduced an issue where if
            # the playbook is executed from within a directory without
            # read / write permission (which can happen when you,
            # for example, execute via sudo) it will fail. the
            # workaround will be to run the ansible command from /tmp
            # and then change back to the original directory at the end
            current_dir = os.getcwd()  # nosec
            os.chdir('/tmp')  # nosec
            self._process = subprocess.Popen(self._command,  # nosec
                                             shell=True,
                                             stdout=subprocess.PIPE,
                                             stderr=subprocess.PIPE)

            # setup stdout to be read without blocking
            LOG.debug('process pid: %s' % self._process.pid)
            flags = fcntl.fcntl(self._process.stdout, fcntl.F_GETFL)
            fcntl.fcntl(self._process.stdout, fcntl.F_SETFL,
                        (flags | os.O_NONBLOCK))

            # this is also part of the fix for ansible 2.2 and later
            os.chdir(current_dir)
        except Exception as e:
            self._cleanup()
            raise e

    def wait(self):
        """wait for job to complete

        return status of job (see get_status for status values)
        """
        while True:
            status = self.get_status()
            if status is not None:
                break
            time.sleep(0.2)
        return status

    def get_status(self):
        """get process status

        status:
        - None: running
        - 0: done, success
        - 1: done, error
        - 2: done, killed by user
        """
        status = self._process.poll()
        out = self._read_stream(self._process.stdout)
        self._cmd_output = ''.join([self._cmd_output, out])

        # unnecessary spaces should be hidden
        if out:
            self._log_output(out)
        if status is not None:
            # job has completed
            if self._kill_uname:
                status = 2
                msg = (u._('Job killed by user ({name})')
                       .format(name=self._kill_uname))
                self._errors = [msg]
            else:
                status = self._process.returncode
                if status != 0:
                    # if the process ran and returned a non zero return
                    # code we want to see if we got some ansible errors
                    # and if so if we ignored all the errors. if all
                    # errors are ignored we consider the job a success
                    if (self._error_total > 0 and
                            self._error_total == self._ignore_total):
                        status = 0
                    else:
                        status = 1
            if not self._process_std_err:
                # read stderr from process
                std_err = self._read_stream(self._process.stderr)
                self._process_std_err = std_err.strip()
            self._cleanup()
        return status

    def get_error_message(self):
        """"get error message"""
        msg = ''
        for error in self._errors:
            if error:
                msg = ''.join([msg, error, '\n'])

        if ANSIBLE_1_OR_MORE in msg:
            msg = self._get_msg_from_cmdout(msg)
        if not msg:
            msg = self._process_std_err
        return msg

    def get_command_output(self):
        """get command output

        get final output text from command execution
        """
        return self._cmd_output

    def kill(self):
        """kill job in progress

        The process pid is owned by root, so
        that is not killable. Need to kill all its children.
        """
        # the kill must be run as the kolla user so the
        # kolla_actions program must be used.
        try:
            actions_path = get_kolla_actions_path()
            cmd_prefix = ('%s job -t -p '
                          % actions_path)

            # kill the children from largest to smallest pids.
            child_pids = PidManager.get_child_pids(self._process.pid)
            for child_pid in sorted(child_pids, reverse=True):
                cmd = ''.join([cmd_prefix, child_pid])
                err_msg, output = run_cmd(cmd, print_output=False)
                if err_msg:
                    LOG.debug('kill failed: %s %s' % (err_msg, output))
                else:
                    LOG.debug('kill succeeded: %s' % child_pid)

            # record the name of user who killed the job
            cur_uid = os.getuid()
            self._kill_uname = pwd.getpwuid(cur_uid)[0]
        finally:
            self._cleanup()

    def _get_msg_from_cmdout(self, msg):
        """get message from command output

        This is where the error message is in cmd out-
        \nfailed: [ol7-c5] (item=[u'/etc/kolla/config/aodh.conf',
        u'/usr/share/kolla/templates/aodh/aodh.conf_augment']) =>
        {"failed": true, "invocation": {"module_args": {"dest":
        "/usr/share/kolla/templates/aodh/aodh.conf_augment",
        "src": "/etc/kolla/config/aodh.conf"}, "module_name": "template"},
        "item": ["/etc/kolla/config/aodh.conf",
        "/usr/share/kolla/templates/aodh/aodh.conf_augment"],
        "msg": "IOError: [Errno 2] No such file or directory:
        u'/etc/kolla/config/aodh.conf'"}\n
        """
        fail_key = '\nfailed: '
        hostnames = re.findall(fail_key + r'\[(.+?)]', self._cmd_output)
        msgs = re.findall(fail_key + '.+ => (.+?)\n', self._cmd_output)

        for i in range(0, min(len(hostnames), len(msgs))):
            err = ''
            hostname = hostnames[i]
            ans_dict_str = msgs[i]
            try:
                ans_dict = json.loads(ans_dict_str)
                err = ans_dict.get('msg', '')
            except Exception as e:
                LOG.warn('Exception reading cmd_out ansible dictionary: %s'
                         % str(e))
            msg = ''.join([msg, 'Host: ', hostname, ', ', err, '\n'])
        return msg

    def _read_stream(self, stream):
        out = ''
        if stream and not stream.closed:
            try:
                out = safe_decode(stream.read())
            except IOError:  # nosec
                # error can happen if stream is empty
                pass
            if out is None:
                out = ''
        return out

    def _log_lines(self, lines):
        if self._print_output:
            for line in lines:
                LOG.info(line)

    def _log_output(self, output):
        if self._print_output:
            LOG.info(output)

    def _cleanup(self):
        """cleanup job

        - release the ansible lock
        - close stdout and stderr
        - delete temp inventory
        """
        # try to clear the ansible lock
        self._ansible_lock.release()

        # close the process's stdout and stderr streams
        if (self._process and self._process.stdout and not
           self._process.stdout.closed):
            self._process.stdout.close()
        if (self._process and self._process.stderr and not
           self._process.stderr.closed):
            self._process.stderr.close()

        # delete temp inventory file
        remove_temp_inventory(self._temp_inv_path)



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Exception definitions."""
import kolla_cli.i18n as u


class CommandError(Exception):
    """CLI command error"""
    def __init__(self, message, *args):
        prefix = u._('ERROR: ')
        if not message.startswith(prefix):
            message = prefix + message
        super(CommandError, self).__init__(message, *args)
